-----------------------------------------
-----------------------------------------
spool C:\UOC\PFC\mbernard\logs\ap1-package_auxiliar.log
-----------------------------------------
-----------------------------------------

CREATE OR REPLACE PACKAGE  pfc_BDR.PKG_AUXILIAR AS
/******************************************************************************
   NOM:   PKG_AUXILIAR
   OBJECTIU: GESTIO DE TOTA LA INFORMACI RELACIONADA AMB TAULES AUXILIARS
			 - POBLACIONS, USUARIS, PRIORITATS
******************************************************************************/

  /**************************************
   *  MANTENIMENT POBLACIONS
   **************************************/

  /******************************************************************************
   NOM:      PR_ALTA_POBLACIO
   OBJECTIU:   Dna d'alta una poblacio, si supera els segents controls:
                  -Tots els camps obligatoris tenen valor
                  -El registre no ha estat inserit previament amb un altre ID
   PARMETRES:
        INPUT:  Obligatoris: nompoblacio: Nom de la poblacio
                Opcionals:   
				No calen: dataAlta s la data d'alta en el sistema 
								 i pren per valor la data del sistema en el moment de la inserci
						  dataBaixa no cal ja que estem donant d'alta un centre
						  idpoblacio s'emplena automticament amb un autonumric en inserir-se
       OUTPUT:  RES
   PRECONDICI: No existeix la poblacio a crear
   POSTCONDICI: S'ha donat d'alta la poblacio
   RETORNA: 
        Si ha anat b:   
                'OK' si l'operaci s'ha realitzat amb xit
        Si no ha anat b algun dels missatges segents, depenent de l'error:   
                'ERROR: El camp no pot ser nul'
                'ERROR: La poblacio ja ha estat donada d'alta'
                'ERROR: Tipus d Error' en qualsevol altre cas 
   ******************************************************************************/
  PROCEDURE PR_ALTA_POBLACIO (p_nompoblacio  IN poblacio.nompoblacio%TYPE,
                               RES             OUT VARCHAR2);


  /******************************************************************************
   NOM:      PR_BAIXA_POBLACIO
   OBJECTIU:   Dna de baixa una poblacio, si supera els segents controls:
                 -Tots els camps obligatoris tenen valor
                 -La poblacio existeix en la Base de Dades
                 -La poblacio no est donada de baixa
   PARMETRES:
        INPUT:  Obligatoris: p_idpoblacio: Identificador de la poblacio
        OUTPUT:  RES
   PRECONDICI: Existeix la poblacio a donar de baixa
   POSTCONDICI: Es dna de baixa la poblacio        
   RETORNA:    
        Si ha anat b:   
                'OK' si l'operaci s'ha realitzat amb xit
        Si no ha anat b algun dels missatges segents, depenent de l'error:   
                'ERROR: La poblacio a donar de baixa no existeix en la base de dades'
                'ERROR: La poblacio ja havia estat donada de baixa'
                'ERROR: El camp ID poblacio no pot ser nul'
				'ERROR: La data de baixa no pot ser inferior a la data d alta'
                'ERROR: Tipus d Error' en qualsevol altre cas    
   ******************************************************************************/
   PROCEDURE PR_BAIXA_POBLACIO (p_idpoblacio  IN poblacio.idpoblacio%TYPE,
								 RES            OUT VARCHAR2);

 
  /******************************************************************************
   NOM:      PR_MODIF_POBLACIO
   OBJECTIU:   Realitza una modificaci en una poblacio de qualsevol dels parmetres opcionals,
                si supera els segents controls:
                 -Tots els camps obligatoris tenen valor
                 -la poblacio existeix en la Base de Dades              
   PARMETRES:
        INPUT:  Obligatoris: p_idpoblacio
							 p_nompoblacio
                Opcionals:   p_dataAlta
							 p_dataBaixa
        OUTPUT:  RES
   PRECONDICI: Existeix la poblacio a modificar
   POSTCONDICI: Es modifica la poblacio                
   RETORNA: 
        Si ha anat b:   
                'OK' si l'operaci s'ha realitzat amb xit
        Si no ha anat b algun dels missatges segents, depenent de l'error:   
                'ERROR: L identificador de la poblacio a modificar no existeix'
                'ERROR: L identificador de la poblacio no pot ser nul'
                'ERROR: La data d alta no pot ser posterior a la data de baixa'
                'ERROR: La data de baixa no pot ser anterior a la data d alta'
                'ERROR: Tipus d Error' en qualsevol altre cas 
   ******************************************************************************/
   PROCEDURE PR_MODIF_POBLACIO (p_idpoblacio   IN poblacio.idpoblacio%TYPE,
                              p_nompoblacio  IN poblacio.nompoblacio%TYPE,
                              p_dataalta     IN poblacio.dataAlta%TYPE,
                              p_databaixa    IN poblacio.dataBaixa%TYPE,
                              RES             OUT VARCHAR2);
							  

   FUNCTION FN_OBTENIR_ID_NOMPOBLACIO(p_nomPoblacio POBLACIO.nomPoblacio%TYPE) 
   RETURN POBLACIO.idpoblacio%TYPE;

   
  /**************************************
   *  MANTENIMENT USUARIS
   **************************************/	
  PROCEDURE PR_ALTA_USUARI (p_login  	IN USUARI.login%TYPE,
							p_pwd  		IN USUARI.pwd%TYPE,
							p_dni  		IN USUARI.dni%TYPE,
							p_nom  		IN USUARI.nom%TYPE,
							p_mail 		IN USUARI.mail%TYPE,
							p_perfil  	IN USUARI.perfil%TYPE,
							RES 		OUT VARCHAR2);
							
							
  PROCEDURE PR_BAIXA_USUARI (p_idusuari IN USUARI.idusuari%TYPE,
							RES     	OUT VARCHAR2);
							
							
  PROCEDURE PR_MODIF_USUARI (p_idusuari IN USUARI.idusuari%TYPE,
							p_login  	IN USUARI.login%TYPE,
							p_pwd  		IN USUARI.pwd%TYPE,
							p_dni  		IN USUARI.dni%TYPE,
							p_nom  		IN USUARI.nom%TYPE,
							p_mail 		IN USUARI.mail%TYPE,
							p_perfil  	IN USUARI.perfil%TYPE,				
							p_dataalta  IN USUARI.dataAlta%TYPE,
							p_databaixa IN USUARI.dataBaixa%TYPE,
							RES     	OUT VARCHAR2);				  
 
 
  FUNCTION FN_OBTENIR_ID_LOGIN_USUARI(p_login USUARI.login%TYPE) 
  RETURN USUARI.idusuari%TYPE;
   
   
  /**************************************
  *  MANTENIMENT PRIORITATS
  **************************************/
  PROCEDURE PR_ALTA_PRIORITAT (p_gravetat  		IN PRIORITAT.gravetat%TYPE,
							p_qDiesResolucio 	IN PRIORITAT.qDiesResolucio%TYPE,
							RES 				OUT VARCHAR2);
							
							
  PROCEDURE PR_BAIXA_PRIORITAT (p_idprioritat 	IN PRIORITAT.idprioritat%TYPE,
								RES     		OUT VARCHAR2);
							
							
  PROCEDURE PR_MODIF_PRIORITAT (p_idprioritat 		IN PRIORITAT.idprioritat%TYPE,
								p_gravetat  		IN PRIORITAT.gravetat%TYPE,
								p_qDiesResolucio 	IN PRIORITAT.qDiesResolucio%TYPE,			
								p_dataalta  		IN PRIORITAT.dataAlta%TYPE,
								p_databaixa 		IN PRIORITAT.dataBaixa%TYPE,
								RES     			OUT VARCHAR2);	
 
END;
/


CREATE OR REPLACE
PACKAGE BODY pfc_BDR.PKG_AUXILIAR AS

  /**************************************
   *  MANTENIMENT POBLACIONS
   **************************************/

  FUNCTION FN_OBTENIR_ID_NOMPOBLACIO(p_nomPoblacio POBLACIO.nomPoblacio%TYPE)
  RETURN POBLACIO.idpoblacio%TYPE
  IS
	idAux POBLACIO.idpoblacio%TYPE; 
	q INTEGER;
  BEGIN

	  SELECT count(*) as d  
	  INTO q
	  FROM POBLACIO
	  WHERE p_nomPoblacio = nomPoblacio;

	  IF (q > 0) THEN
		  SELECT idpoblacio 
			INTO idAux
		  FROM POBLACIO
		  WHERE p_nomPoblacio = nomPoblacio;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
  END;


  PROCEDURE PR_ALTA_POBLACIO (p_nompoblacio  IN poblacio.nompoblacio%TYPE,
                               RES             OUT VARCHAR2)
  IS
   E_CAMP_NUL         EXCEPTION;
   E_POBLACIO_EXISTEIX      EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NOMpoblacio: '||p_nompoblacio;

      --Verifica que tots els camps obligatoris tenen valor
      IF (TRIM(p_nompoblacio) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que el registre no s'hagi inserit previament
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM poblacio
        WHERE nompoblacio  = p_nompoblacio
          AND ROWNUM  = 1;

        RAISE E_POBLACIO_EXISTEIX;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
      END;

      --Si ha passat les proves de validaci realitza la inserci
      INSERT INTO poblacio (nompoblacio)
           VALUES (p_nompoblacio);

      RES := 'OK';
  
      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_AUXILIAR.PR_ALTA_POBLACIO', param_entrada, 'RES: '||RES);
      COMMIT;

  EXCEPTION
    WHEN E_POBLACIO_EXISTEIX THEN
      RES := 'ERROR: la poblacio ja ha estat donada d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_POBLACIO', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_POBLACIO', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_POBLACIO', param_entrada, 'RES: '||RES);
      COMMIT;

  END;

  
  PROCEDURE PR_BAIXA_POBLACIO (p_idpoblacio    IN poblacio.idpoblacio%TYPE,
                               RES             OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_POBLACIO_NOEXISTEIX EXCEPTION;
   E_POBLACIO_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


  BEGIN

      param_entrada := 'P_idpoblacio: '||p_idpoblacio;

      --Verifica que tots els camps obligatoris tenen valor
      IF (TRIM(p_idpoblacio) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que la poblacio existeix en la Base de Dades
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM poblacio
        WHERE idpoblacio = p_idpoblacio
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_POBLACIO_NOEXISTEIX;
      END;

      --Verifica que la poblacio no est donada de baixa
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM poblacio
        WHERE idpoblacio = p_idpoblacio
          AND dataBaixa  IS NULL
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_POBLACIO_JA_BAIXA;
      END;

      --Si ha passat les proves de validaci realitza l'esborrat lgic de la poblacio
      UPDATE poblacio SET dataBaixa = SYSDATE
      WHERE  idpoblacio = p_idpoblacio;

      RES := 'OK';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_AUXILIAR.PR_BAIXA_POBLACIO', param_entrada, 'RES: '||RES);
      COMMIT;

   EXCEPTION
      WHEN E_POBLACIO_NOEXISTEIX THEN
        RES := 'ERROR: La poblacio a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN E_POBLACIO_JA_BAIXA THEN
        RES := 'ERROR: La poblacio ja havia estat donada de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp ID poblacio no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

  END;

  
  PROCEDURE PR_MODIF_POBLACIO (p_idpoblacio   IN poblacio.idpoblacio%TYPE,
								p_nompoblacio  IN poblacio.nompoblacio%TYPE,
								p_dataalta     IN poblacio.dataAlta%TYPE,
								p_databaixa    IN poblacio.dataBaixa%TYPE,
								RES             OUT VARCHAR2)
  IS
   E_CAMP_NUL              EXCEPTION;
   E_POBLACIO_NOEXISTEIX   EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_idpoblacio: '||p_idpoblacio||', P_nompoblacio: '||p_nompoblacio||', P_DataALTA: '||p_dataAlta||', P_Databaixa: '||p_databaixa;

      --Verifica que tots els camps obligatoris tenen valor
      IF (TRIM(p_idpoblacio) IS NULL) OR (TRIM(p_nompoblacio) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que la poblacio existeix en la Base de Dades
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM poblacio
        WHERE idpoblacio = p_idpoblacio
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_POBLACIO_NOEXISTEIX;
      END;

		-- si es modifica la data d'alta
       IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > p_databaixa) THEN
             RAISE E_DATA_BAIXA_INCORRECTA;
         END IF;
       END IF;
	   
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;
    
     --Si es passen les proves de validaci actualiza
     UPDATE poblacio
        SET  nompoblacio = nvl(p_nompoblacio, nompoblacio),
             dataAlta    = nvl(p_dataalta, dataAlta),
             dataBaixa     = nvl(p_databaixa, dataBaixa)
      WHERE   idpoblacio = p_idpoblacio;
      commit;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION

      WHEN E_POBLACIO_NOEXISTEIX THEN
        RES := 'ERROR: L identificador de la poblacio a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: L identificador de la poblacio no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;
		
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;		

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_POBLACIO', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
         
		 
  /**************************************
   *  MANTENIMENT USUARIS
   **************************************/	
  FUNCTION FN_OBTENIR_ID_LOGIN_USUARI(p_login USUARI.login%TYPE) 
  RETURN USUARI.idusuari%TYPE
  IS
	idAux USUARI.idusuari%TYPE; 
	q INTEGER;
  BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM USUARI
	  WHERE p_login = login;

	  IF (q > 0) THEN
		  SELECT idusuari 
			INTO idAux
		  FROM USUARI
		  WHERE p_login = login;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
  END;
  
   
  PROCEDURE PR_ALTA_USUARI (p_login  	IN USUARI.login%TYPE,
							p_pwd  		IN USUARI.pwd%TYPE,
							p_dni  		IN USUARI.dni%TYPE,
							p_nom  		IN USUARI.nom%TYPE,
							p_mail 		IN USUARI.mail%TYPE,
							p_perfil  	IN USUARI.perfil%TYPE,
							RES 		OUT VARCHAR2)
  IS
   E_CAMP_NUL            EXCEPTION;
   E_USUARI_EXISTEIX     EXCEPTION;
   E_PERFIL_INCORRECTE   EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_LOGIN: '||p_login||', P_PWD: '||p_pwd||', P_DNI: '||p_dni||', P_NOM: '||p_nom||', P_MAIL: '||p_mail||', P_PERFIL: '||p_perfil;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_login) IS NULL) OR (TRIM(p_pwd) IS NULL) OR (TRIM(p_dni) IS NULL) 
			OR (TRIM(p_nom) IS NULL) OR (TRIM(p_perfil) IS NULL)
		THEN 
			RAISE E_CAMP_NUL ;
     END IF;
	 
	 --Verifica que el perfil sigui correcte
     IF ((TRIM(p_perfil) != 'ADMINISTRADOR') AND (TRIM(p_perfil) != 'REFERENT')
			 AND (TRIM(p_perfil) != 'LOGISTIC') AND (TRIM(p_perfil) != 'ANALISTA'))
		THEN 
			RAISE E_PERFIL_INCORRECTE;
     END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM USUARI
        WHERE login  = p_login
			  AND ROWNUM  = 1;

        RAISE E_USUARI_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

     --Si ha passat les proves de validaci realitza la inserci
     INSERT INTO USUARI (login,pwd,dni,nom,mail,perfil)
          VALUES (p_login,p_pwd,p_dni,p_nom,p_mail,p_perfil);

     RES := 'OK';
  
     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_AUXILIAR.PR_ALTA_USUARI', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
    WHEN E_USUARI_EXISTEIX THEN
      RES := 'ERROR: l usuari ja ha estat donat d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_USUARI', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_USUARI', param_entrada, 'RES: '||RES);
      COMMIT;
	  
	WHEN E_PERFIL_INCORRECTE  THEN
      RES := 'ERROR: El perfil noms pot ser ADMINISTRADOR, LOGISTIC, REFERENT o ANALISTA';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_USUARI', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_USUARI', param_entrada, 'RES: '||RES);
      COMMIT;
  END;
							
							
  PROCEDURE PR_BAIXA_USUARI (p_idusuari IN USUARI.idusuari%TYPE,
							RES     	OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_USUARI_NOEXISTEIX EXCEPTION;
   E_USUARI_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

      param_entrada := 'P_IDUSUARI: '||p_idusuari;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idusuari) IS NULL) 
		 THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l usuari existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM USUARI
        WHERE idusuari = p_idusuari
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_USUARI_NOEXISTEIX;
     END;

     --Verifica que l usuari no est donat de baixa
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM USUARI
        WHERE idusuari = p_idusuari
			AND dataBaixa  IS NULL
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_USUARI_JA_BAIXA;
     END;

     --Si ha passat les proves de validaci realitza l'esborrat lgic de l usuari
     UPDATE USUARI SET dataBaixa = SYSDATE
     WHERE  idusuari = p_idusuari;

     RES := 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_AUXILIAR.PR_BAIXA_USUARI', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_USUARI_NOEXISTEIX THEN
        RES := 'ERROR: L usuari a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_USUARI_JA_BAIXA THEN
        RES := 'ERROR: L usuari ja havia estat donat de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
							
							
  PROCEDURE PR_MODIF_USUARI (p_idusuari IN USUARI.idusuari%TYPE,
							p_login  	IN USUARI.login%TYPE,
							p_pwd  		IN USUARI.pwd%TYPE,
							p_dni  		IN USUARI.dni%TYPE,
							p_nom  		IN USUARI.nom%TYPE,
							p_mail 		IN USUARI.mail%TYPE,
							p_perfil  	IN USUARI.perfil%TYPE,				
							p_dataalta  IN USUARI.dataAlta%TYPE,
							p_databaixa IN USUARI.dataBaixa%TYPE,
							RES     	OUT VARCHAR2)											   
  IS
   E_CAMP_NUL              EXCEPTION;
   E_USUARI_NOEXISTEIX     EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_PERFIL_INCORRECTE   EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;   

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDUSUARI: '||p_idusuari||', P_LOGIN: '||p_login||', P_PWD: '||p_pwd||', P_DNI: '||p_dni||', P_NOM: '||p_nom||', P_MAIL: '||p_mail||', P_PERFIL: '||p_perfil||', P_DATAALTA: '||p_dataalta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idusuari) IS NULL) OR (TRIM(p_login) IS NULL) OR (TRIM(p_pwd) IS NULL) OR (TRIM(p_dni) IS NULL) 
			OR (TRIM(p_nom) IS NULL) OR (TRIM(p_perfil) IS NULL) 
		THEN 
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l usuari existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM USUARI
        WHERE idusuari = p_idusuari
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_USUARI_NOEXISTEIX;
     END;

	 -- si es modifica la data d'alta
	 IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
		IF (p_dataalta > p_databaixa) THEN
				RAISE E_DATA_BAIXA_INCORRECTA;
		END IF;
	 END IF;
	 
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;	 
	 
	 --Verifica que el perfil sigui correcte
     IF ((TRIM(p_perfil) != 'ADMINISTRADOR') AND (TRIM(p_perfil) != 'REFERENT')
			 AND (TRIM(p_perfil) != 'LOGISTIC') AND (TRIM(p_perfil) != 'ANALISTA'))
		THEN 
			RAISE E_PERFIL_INCORRECTE;
     END IF;
    
     --Si es passen les proves de validaci actualizem
     UPDATE USUARI
        SET login = nvl(p_login, login),
			pwd = nvl(p_pwd, pwd),
			dni = nvl(p_dni, dni),
			nom = nvl(p_nom, nom),
			mail = nvl(p_mail, mail),
			perfil = nvl(p_perfil, perfil),
            dataAlta    = nvl(p_dataalta, dataAlta),
            dataBaixa   = nvl(p_databaixa, dataBaixa)
     WHERE   idusuari = p_idusuari;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_USUARI_NOEXISTEIX THEN
        RES := 'ERROR: L usuari a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;
		
	WHEN E_PERFIL_INCORRECTE  THEN
      RES := 'ERROR: El perfil noms pot ser ADMINISTRADOR, LOGISTIC, REFERENT o ANALISTA';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
      COMMIT;		
	  
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;		  

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_USUARI', param_entrada, 'RES: '||RES);
        COMMIT;
  END;


   /**************************************
   *  MANTENIMENT PRIORITATS
   **************************************/
  PROCEDURE PR_ALTA_PRIORITAT (p_gravetat  		IN PRIORITAT.gravetat%TYPE,
							p_qDiesResolucio 	IN PRIORITAT.qDiesResolucio%TYPE,
							RES 				OUT VARCHAR2)
  IS
   E_CAMP_NUL         	EXCEPTION;
   E_PRIORITAT_EXISTEIX     EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_GRAVETAT: '||p_gravetat||', P_QDIESRESOLUCIO: '||p_qDiesResolucio;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_gravetat) IS NULL) OR (TRIM(p_qDiesResolucio) IS NULL)
		THEN 
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM PRIORITAT
        WHERE gravetat  = p_gravetat
			  AND ROWNUM  = 1;

        RAISE E_PRIORITAT_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

     --Si ha passat les proves de validaci realitza la inserci
     INSERT INTO PRIORITAT (gravetat,qDiesResolucio)
          VALUES (p_gravetat,p_qDiesResolucio);

     RES := 'OK';
  
     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_AUXILIAR.PR_ALTA_PRIORITAT', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
    WHEN E_PRIORITAT_EXISTEIX THEN
      RES := 'ERROR: La prioritat ja ha estat donada d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_PRIORITAT', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_PRIORITAT', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_AUXILIAR.PR_ALTA_PRIORITAT', param_entrada, 'RES: '||RES);
      COMMIT;
  END;
							
							
  PROCEDURE PR_BAIXA_PRIORITAT (p_idprioritat 	IN PRIORITAT.idprioritat%TYPE,
								RES     		OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_PRIORITAT_NOEXISTEIX EXCEPTION;
   E_PRIORITAT_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDPRIORITAT: '||p_idprioritat;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idprioritat) IS NULL) 
		 THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l usuari existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM PRIORITAT
        WHERE idprioritat = p_idprioritat
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_PRIORITAT_NOEXISTEIX;
     END;

     --Verifica que l usuari no est donat de baixa
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM PRIORITAT
        WHERE idprioritat = p_idprioritat
			AND dataBaixa  IS NULL
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_PRIORITAT_JA_BAIXA;
     END;

     --Si ha passat les proves de validaci realitza l'esborrat lgic de l usuari
     UPDATE PRIORITAT SET dataBaixa = SYSDATE
     WHERE  idprioritat = p_idprioritat;

     RES := 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_AUXILIAR.PR_BAIXA_PRIORITAT', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_PRIORITAT_NOEXISTEIX THEN
        RES := 'ERROR: La prioritat a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_PRIORITAT_JA_BAIXA THEN
        RES := 'ERROR: La prioritat ja havia estat donada de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_BAIXA_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
  
							
  PROCEDURE PR_MODIF_PRIORITAT (p_idprioritat 		IN PRIORITAT.idprioritat%TYPE,
								p_gravetat  		IN PRIORITAT.gravetat%TYPE,
								p_qDiesResolucio 	IN PRIORITAT.qDiesResolucio%TYPE,			
								p_dataalta  		IN PRIORITAT.dataAlta%TYPE,
								p_databaixa 		IN PRIORITAT.dataBaixa%TYPE,
								RES     			OUT VARCHAR2)	  
  IS
   E_CAMP_NUL              EXCEPTION;
   E_PRIORITAT_NOEXISTEIX     EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDPRIORITAT: '||p_idprioritat||', P_GRAVETAT: '||p_gravetat||', P_QDIESRESOLUCIO: '||p_qDiesResolucio||', P_DATAALTA: '||p_dataalta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idprioritat) IS NULL) OR (TRIM(p_gravetat) IS NULL) OR (TRIM(p_qDiesResolucio) IS NULL) 
		THEN 
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l usuari existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM PRIORITAT
        WHERE idprioritat = p_idprioritat
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_PRIORITAT_NOEXISTEIX;
     END;

	 -- si es modifica la data d'alta
	 IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
		IF (p_dataalta > p_databaixa) THEN
				RAISE E_DATA_BAIXA_INCORRECTA;
		END IF;
	 END IF;
    
     --Si es passen les proves de validaci actualizem
     UPDATE PRIORITAT
        SET gravetat = nvl(p_gravetat, gravetat),
			qDiesResolucio = nvl(p_qDiesResolucio, qDiesResolucio),
            dataAlta    = nvl(p_dataalta, dataAlta),
            dataBaixa   = nvl(p_databaixa, dataBaixa)
     WHERE   idprioritat = p_idprioritat;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_AUXILIAR.PR_MODIF_PRIORITAT', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_PRIORITAT_NOEXISTEIX THEN
        RES := 'ERROR: La prioritat a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_AUXILIAR.PR_MODIF_PRIORITAT', param_entrada, 'RES: '||RES);
        COMMIT;
  END;

  
END;
/

COMMIT;

spool off
exit
