-----------------------------------------
-----------------------------------------
spool C:\UOC\PFC\mbernard\logs\ap2-package_centres.log
-----------------------------------------
-----------------------------------------

CREATE OR REPLACE PACKAGE pfc_BDR.PKG_CENTRES AS
/******************************************************************************
   NOM:   PKG_CENTRES
   OBJECTIU: GESTIO DE TOTA LA INFORMACI RELACIONADA AMB ELS CENTRES
******************************************************************************/

  /**************************************
   *  MANTENIMENT CENTRES
   **************************************/

  PROCEDURE PR_ALTA_CENTRE (p_nomcentre  IN centre.nomcentre%TYPE,
                            p_adrea IN centre.adrea%TYPE,
							p_idpoblacio IN centre.idpoblacio%TYPE,
							p_codipostal IN centre.codipostal%TYPE,
							p_telefon IN centre.telefon%TYPE,
							p_mail IN centre.mail%TYPE,
							RES OUT VARCHAR2); 
		  /******************************************************************************
		   NOM:      PR_ALTA_centre
		   OBJECTIU:   Dna d'alta una centre, si supera els segents controls:
						  -Tots els camps obligatoris tenen valor
						  -El registre no ha estat inserit previament amb un altre ID
		   PARMETRES:
				INPUT:  Obligatoris: nomcentre: Nom del centre
						Opcionals:   
						No calen: dataAlta s la data d'alta en el sistema 
										 i pren per valor la data del sistema en el moment de la inserci
								  dataBaixa no cal ja que estem donant d'alta un centre
								  idcentre s'emplena automticament amb un autonumric en inserir-se
			   OUTPUT:  RES
		   PRECONDICI: No existeix el centre a crear
		   POSTCONDICI: S'ha donat d'alta la centre
		   RETORNA: 
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: El camp no pot ser nul'
						'ERROR: La centre ja ha estat donada d'alta'
						'ERROR: Tipus d Error' en qualsevol altre cas 
		   ******************************************************************************/							
							
							
  PROCEDURE PR_BAIXA_centre (p_idcentre  IN centre.idcentre%TYPE,
							RES          OUT VARCHAR2);							
		  /******************************************************************************
		   NOM:      PR_BAIXA_centre
		   OBJECTIU:   Dna de baixa una centre, si supera els segents controls:
						 -Tots els camps obligatoris tenen valor
						 -La centre existeix en la Base de Dades
						 -La centre no est donada de baixa
		   PARMETRES:
				INPUT:  Obligatoris: p_idcentre: Identificador de la centre
				OUTPUT:  RES
		   PRECONDICI: Existeix la centre a donar de baixa
		   POSTCONDICI: Es dna de baixa el centre        
		   RETORNA:    
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: La centre a donar de baixa no existeix en la base de dades'
						'ERROR: La centre ja havia estat donada de baixa'
						'ERROR: El camp ID centre no pot ser nul'
						'ERROR: La data de baixa no pot ser inferior a la data d alta'
						'ERROR: Tipus d Error' en qualsevol altre cas    
		   ******************************************************************************/


  PROCEDURE PR_MODIF_centre (p_idcentre   IN centre.idcentre%TYPE,
							p_nomcentre  IN centre.nomcentre%TYPE,
                            p_adrea IN centre.adrea%TYPE,
							p_idpoblacio IN centre.idpoblacio%TYPE,
							p_codipostal IN centre.codipostal%TYPE,
							p_telefon IN centre.telefon%TYPE,
							p_mail IN centre.mail%TYPE,
							p_dataalta     IN centre.dataAlta%TYPE,
                            p_databaixa    IN centre.dataBaixa%TYPE,
							RES OUT VARCHAR2);  
		  /******************************************************************************
		   NOM:      PR_MODIF_centre
		   OBJECTIU:   Realitza una modificaci en una centre de qualsevol dels parmetres opcionals,
						si supera els segents controls:
						 -Tots els camps obligatoris tenen valor
						 -la centre existeix en la Base de Dades              
		   PARMETRES:
				INPUT:  Obligatoris: p_idcentre
									 p_nomcentre
						Opcionals:   p_dataAlta
									 p_dataBaixa
				OUTPUT:  RES
		   PRECONDICI: Existeix la centre a modificar
		   POSTCONDICI: Es modifica la centre                
		   RETORNA: 
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: L identificador de la centre a modificar no existeix'
						'ERROR: L identificador de la centre no pot ser nul'
						'ERROR: La data d alta no pot ser posterior a la data de baixa'
						'ERROR: La data de baixa no pot ser anterior a la data d alta'
						'ERROR: Tipus d Error' en qualsevol altre cas 
		   ******************************************************************************/

							
   FUNCTION FN_OBTENIR_ID_NOMCENTRE(p_nomCentre CENTRE.nomCentre%TYPE) 
   RETURN CENTRE.idcentre%TYPE;						
 
END;
/


CREATE OR REPLACE
PACKAGE BODY   pfc_BDR.PKG_CENTRES AS

  FUNCTION FN_OBTENIR_ID_NOMCENTRE(p_nomCentre CENTRE.nomCentre%TYPE) 
  RETURN CENTRE.idcentre%TYPE
  IS
	idAux CENTRE.idcentre%TYPE; 
	q INTEGER;
  BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM CENTRE
	  WHERE p_nomCentre = nomCentre;

	  IF (q > 0) THEN
		  SELECT idcentre 
			INTO idAux
		  FROM CENTRE
		  WHERE p_nomCentre = nomCentre;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
  END;

  PROCEDURE PR_ALTA_centre (p_nomcentre  IN centre.nomcentre%TYPE,
                            p_adrea IN centre.adrea%TYPE,
							p_idpoblacio IN centre.idpoblacio%TYPE,
							p_codipostal IN centre.codipostal%TYPE,
							p_telefon IN centre.telefon%TYPE,
							p_mail IN centre.mail%TYPE,
							RES OUT VARCHAR2)
  IS
   E_CAMP_NUL         	  EXCEPTION;
   E_centre_EXISTEIX      EXCEPTION;
   E_poblacio_NOEXISTEIX  EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NOMcentre: '||p_nomcentre;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_nomcentre) IS NULL) OR (TRIM(p_adrea) IS NULL) OR (TRIM(p_idpoblacio) IS NULL) OR (TRIM(p_codipostal) IS NULL) OR (TRIM(p_telefon) IS NULL)
		THEN
         RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que el registre no s'hagi inserit previament
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM CENTRE
        WHERE nomcentre  = p_nomcentre
          AND ROWNUM  = 1;

        RAISE E_centre_EXISTEIX;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
      END;
	  
	  --Verifica que la poblacio existeix en la base de dades
      BEGIN
       SELECT 'S'
         INTO EXISTEIX
         FROM POBLACIO
        WHERE idpoblacio  = p_idpoblacio
          AND ROWNUM  = 1;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_poblacio_NOEXISTEIX;
      END;

      --Si ha passat les proves de validaci realitza la inserci
      INSERT INTO centre (nomcentre,adrea,idpoblacio,codipostal,telefon,mail)
           VALUES (p_nomcentre,p_adrea,p_idpoblacio,p_codipostal,p_telefon,p_mail);

      RES := 'OK';
  
      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_CENTRES.PR_ALTA_CENTRE', param_entrada, 'RES: '||RES);
      COMMIT;

  EXCEPTION
    WHEN E_centre_EXISTEIX THEN
      RES := 'ERROR: el centre ja ha estat donat d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_CENTRES.PR_ALTA_CENTRE', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_CENTRES.PR_ALTA_CENTRE', param_entrada, 'RES: '||RES);
      COMMIT;
	  
    WHEN E_poblacio_NOEXISTEIX  THEN
      RES := 'ERROR: La poblacio no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_CENTRES.PR_ALTA_CENTRE', param_entrada, 'RES: '||RES);
      COMMIT;	  

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_CENTRES.PR_ALTA_CENTRE', param_entrada, 'RES: '||RES);
      COMMIT;

  END;

  
  PROCEDURE PR_BAIXA_centre (p_idcentre    IN centre.idcentre%TYPE,
								RES          	 OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_centre_NOEXISTEIX EXCEPTION;
   E_centre_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

      param_entrada := 'P_idcentre: '||p_idcentre;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idcentre) IS NULL)
     THEN
         RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que la centre existeix en la Base de Dades
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM centre
        WHERE idcentre = p_idcentre
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_centre_NOEXISTEIX;
      END;

      --Verifica que la centre no est donada de baixa
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM centre
        WHERE idcentre = p_idcentre
          AND dataBaixa  IS NULL
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_centre_JA_BAIXA;
      END;

      --Si ha passat les proves de validaci realitza l'esborrat lgic de la centre
      UPDATE centre SET dataBaixa = SYSDATE
      WHERE  idcentre = p_idcentre;

      RES := 'OK';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_CENTRES.PR_BAIXA_centre', param_entrada, 'RES: '||RES);
      COMMIT;

  EXCEPTION
      WHEN E_centre_NOEXISTEIX THEN
        RES := 'ERROR: El centre a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_BAIXA_centre', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN E_centre_JA_BAIXA THEN
        RES := 'ERROR: El centre ja havia estat donat de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_BAIXA_centre', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_BAIXA_centre', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_BAIXA_centre', param_entrada, 'RES: '||RES);
        COMMIT;

  END;

  
  PROCEDURE PR_MODIF_CENTRE (p_idcentre   IN centre.idcentre%TYPE,
							p_nomcentre  IN centre.nomcentre%TYPE,
							p_adrea IN centre.adrea%TYPE,
							p_idpoblacio IN centre.idpoblacio%TYPE,
							p_codipostal IN centre.codipostal%TYPE,
							p_telefon IN centre.telefon%TYPE,
							p_mail IN centre.mail%TYPE,
							p_dataalta     IN centre.dataAlta%TYPE,
							p_databaixa    IN centre.dataBaixa%TYPE,
							RES             OUT VARCHAR2)
  IS
   E_CAMP_NUL             EXCEPTION;
   E_centre_NOEXISTEIX       EXCEPTION;
   E_poblacio_NOEXISTEIX  EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDCENTRE: '||p_idcentre||', P_NOMCENTRE: '||p_nomcentre||', P_ADREA: '||p_adrea||', P_IDPOBLACIO: '||p_idpoblacio||', P_CODIPOSTAL: '||p_codipostal||', P_TELEFON: '||p_telefon||', P_MAIL: '||p_mail||', P_DATAALTA: '||p_dataAlta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idcentre) IS NULL) OR (TRIM(p_nomcentre) IS NULL) OR (TRIM(p_adrea) IS NULL) 
			OR (TRIM(p_idpoblacio) IS NULL) OR (TRIM(p_codipostal) IS NULL) OR (TRIM(p_telefon) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
      END IF;

     --Verifica que el centre existeix en la Base de Dades
     BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM centre
        WHERE idcentre = p_idcentre
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_centre_NOEXISTEIX;
      END;
	  
	  --Verifica que la poblacio existeix en la base de dades
      BEGIN
       SELECT 'S'
         INTO EXISTEIX
         FROM POBLACIO
        WHERE idpoblacio  = p_idpoblacio
          AND ROWNUM  = 1;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_poblacio_NOEXISTEIX;
      END;

		-- si es modifica la data d alta
       IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > p_databaixa) THEN
             RAISE E_DATA_BAIXA_INCORRECTA;
         END IF;
       END IF;
	   
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;	   
    
     --Si es passen les proves de validaci actualiza
     UPDATE centre
        SET nomcentre = nvl(p_nomcentre, nomcentre),
			adrea = nvl(p_adrea, adrea),
			idpoblacio = nvl(p_idpoblacio, idpoblacio),
			codipostal = nvl(p_codipostal, codipostal),
			telefon = nvl(p_telefon, telefon),
			mail = nvl(p_mail, mail),
            dataAlta  = nvl(p_dataalta, dataAlta),
            dataBaixa = nvl(p_databaixa, dataBaixa)
      WHERE   idcentre = p_idcentre;
      commit;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_CENTRES.PR_MODIF_CENTRE', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION

      WHEN E_centre_NOEXISTEIX THEN
        RES := 'ERROR: El centre a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_centre', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_centre', param_entrada, 'RES: '||RES);
        COMMIT;
		
	  WHEN E_poblacio_NOEXISTEIX  THEN
		RES := 'ERROR: La poblacio no existeix en la base de dades';

		INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
			VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_CENTRE', param_entrada, 'RES: '||RES);
		COMMIT;		

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_centre', param_entrada, 'RES: '||RES);
        COMMIT;
		
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_CENTRE', param_entrada, 'RES: '||RES);
        COMMIT;			

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_CENTRES.PR_MODIF_centre', param_entrada, 'RES: '||RES);
        COMMIT;

  END;
                                                                   

END;
/

COMMIT;

spool off
exit