-----------------------------------------
-----------------------------------------
spool C:\UOC\PFC\mbernard\logs\ap3-package_equips.log
-----------------------------------------
-----------------------------------------

CREATE OR REPLACE PACKAGE pfc_BDR.PKG_EQUIPS AS
/******************************************************************************
   NOM:   PKG_EQUIPS
   OBJECTIU: GESTIO DE TOTA LA INFORMACI RELACIONADA AMB LES CATEGORIES, 
			 ELS MODELS, LES MARQUES I ELS EQUIPS
******************************************************************************/

  /**************************************
   *  MANTENIMENT CATEGORIES
   **************************************/
	PROCEDURE PR_ALTA_CATEGORIA (p_nomCategoria  IN CATEGORIA.nomCategoria%TYPE,
                               RES             OUT VARCHAR2);
		  /******************************************************************************
		   NOM:      PR_ALTA_CATEGORIA
		   OBJECTIU:   Dna d'alta una Categoria, si supera els segents controls:
						  -Tots els camps obligatoris tenen valor
						  -El registre no ha estat inserit previament amb un altre ID
		   PARMETRES:
				INPUT:  Obligatoris: nomCategoria: Nom de la Categoria
						Opcionals:   
						No calen: dataAlta s la data d'alta en el sistema 
										 i pren per valor la data del sistema en el moment de la inserci
								  dataBaixa no cal ja que estem donant d'alta un centre
								  idcategoria s'emplena automticament amb un autonumric en inserir-se
			   OUTPUT:  RES
		   PRECONDICI: No existeix la categoria a crear
		   POSTCONDICI: S'ha donat d'alta la categoria
		   RETORNA: 
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: El camp no pot ser nul'
						'ERROR: La Categoria ja ha estat donada d'alta'
						'ERROR: Tipus d Error' en qualsevol altre cas 
		   ******************************************************************************/
   
   
	PROCEDURE PR_BAIXA_CATEGORIA (p_idcategoria  IN CATEGORIA.idcategoria%TYPE,
								 RES            OUT VARCHAR2);
		  /******************************************************************************
		   NOM:      PR_BAIXA_CATEGORIA
		   OBJECTIU:   Dna de baixa una Categoria, si supera els segents controls:
						 -Tots els camps obligatoris tenen valor
						 -La Categoria existeix en la Base de Dades
						 -La Categoria no est donada de baixa
		   PARMETRES:
				INPUT:  Obligatoris: p_idcategoria: Identificador de la Categoria
				OUTPUT:  RES
		   PRECONDICI: Existeix la categoria a donar de baixa
		   POSTCONDICI: Es dna de baixa la categoria        
		   RETORNA:    
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: La Categoria a donar de baixa no existeix en la base de dades'
						'ERROR: La Categoria ja havia estat donada de baixa'
						'ERROR: El camp ID CATEGORIA no pot ser nul'
						'ERROR: La data de baixa no pot ser inferior a la data d alta'
						'ERROR: Tipus d Error' en qualsevol altre cas    
		   ******************************************************************************/

		   
	PROCEDURE PR_MODIF_CATEGORIA (p_idcategoria   IN CATEGORIA.idcategoria%TYPE,
								p_nomCategoria  IN CATEGORIA.nomCategoria%TYPE,
								p_dataalta     	IN CATEGORIA.dataAlta%TYPE,
								p_databaixa    	IN CATEGORIA.dataBaixa%TYPE,
								RES             OUT VARCHAR2);
		  /******************************************************************************
		   NOM:      PR_MODIF_CATEGORIA
		   OBJECTIU:   Realitza una modificaci en una Categoria de qualsevol dels parmetres opcionals,
						si supera els segents controls:
						 -Tots els camps obligatoris tenen valor
						 -la categoria existeix en la Base de Dades              
		   PARMETRES:
				INPUT:  Obligatoris: p_idcategoria
									 p_nomCategoria
						Opcionals:   p_dataAlta
									 p_dataBaixa
				OUTPUT:  RES
		   PRECONDICI: Existeix la categoria a modificar
		   POSTCONDICI: Es modifica la categoria                
		   RETORNA: 
				Si ha anat b:   
						'OK' si l'operaci s'ha realitzat amb xit
				Si no ha anat b algun dels missatges segents, depenent de l'error:   
						'ERROR: L identificador de la categoria a modificar no existeix'
						'ERROR: L identificador de la categoria no pot ser nul'
						'ERROR: La data d alta no pot ser posterior a la data de baixa'
						'ERROR: La data de baixa no pot ser anterior a la data d alta'
						'ERROR: Tipus d Error' en qualsevol altre cas 
		   ******************************************************************************/

	FUNCTION FN_OBTENIR_ID_NOM_CATEGORIA(p_nomCategoria CATEGORIA.nomCategoria%TYPE) 
	RETURN CATEGORIA.idcategoria%TYPE;
   
								
  /**************************************
   *  MANTENIMENT MARQUES
   **************************************/
	PROCEDURE PR_ALTA_MARCA (p_nomMarca IN MARCA.nomMarca%TYPE,
							RES 	 	OUT VARCHAR2);
													
	PROCEDURE PR_BAIXA_MARCA (p_idmarca IN MARCA.idmarca%TYPE,
							RES         OUT VARCHAR2);
													
	PROCEDURE PR_MODIF_MARCA (p_idmarca IN MARCA.idmarca%TYPE,
							p_nomMarca  IN MARCA.nomMarca%TYPE,
							p_dataalta  IN MARCA.dataAlta%TYPE,
							p_databaixa IN MARCA.dataBaixa%TYPE,
							RES         OUT VARCHAR2);
								
	FUNCTION FN_OBTENIR_ID_NOM_MARCA(p_nomMarca MARCA.nomMarca%TYPE) 
	RETURN MARCA.idmarca%TYPE;
   
   
  /**************************************
   *  MANTENIMENT MODELS
   **************************************/	
	PROCEDURE PR_ALTA_MODEL (p_idmarca  		IN MODEL.idmarca%TYPE,
							p_nomModel  		IN MODEL.nomModel%TYPE,
							p_caracteristiques  IN MODEL.caracteristiques%TYPE,
							p_idcategoria  		IN MODEL.idcategoria%TYPE,
							RES 				OUT VARCHAR2);
													
	PROCEDURE PR_BAIXA_MODEL (p_idmodel IN MODEL.idmodel%TYPE,
							RES     	OUT VARCHAR2);
													
	PROCEDURE PR_MODIF_MODEL (p_idmodel 		IN MODEL.idmodel%TYPE,
							p_nomModel  		IN MODEL.nomModel%TYPE,
							p_caracteristiques  IN MODEL.caracteristiques%TYPE,
							p_idmarca  			IN MODEL.idmarca%TYPE,
							p_idcategoria  		IN MODEL.idcategoria%TYPE,				
							p_dataalta  		IN MODEL.dataAlta%TYPE,
							p_databaixa 		IN MODEL.dataBaixa%TYPE,
							RES     			OUT VARCHAR2);

  	FUNCTION FN_OBTENIR_ID_NOM_MODEL(p_idmarca MODEL.idmarca%TYPE, p_nomModel MODEL.nomModel%TYPE) 
	RETURN MODEL.idmodel%TYPE;

	
  /**************************************
   *  MANTENIMENT EQUIPS
   **************************************/   
	PROCEDURE PR_ALTA_EQUIP (p_numSerie 		IN EQUIP.numSerie%TYPE,
							p_idmodel 			IN EQUIP.idmodel%TYPE,
							p_idcategoria  		IN EQUIP.idcategoria%TYPE,
							p_idcentre  		IN EQUIP.idcentre%TYPE,
							p_descripcio  		IN EQUIP.descripcio%TYPE,
							p_dataCompra  		IN EQUIP.dataCompra%TYPE,
							p_dataFiGarantia	IN EQUIP.dataFiGarantia%TYPE,
							RES 				OUT VARCHAR2);
													
	PROCEDURE PR_BAIXA_EQUIP (p_idequip IN EQUIP.idequip%TYPE,
							RES         OUT VARCHAR2);
													
	PROCEDURE PR_MODIF_EQUIP (p_idequip   		IN EQUIP.idequip%TYPE,
							p_numSerie 			IN EQUIP.numSerie%TYPE,
							p_idmodel 			IN EQUIP.idmodel%TYPE,
							p_idcategoria  		IN EQUIP.idcategoria%TYPE,
							p_idcentre  		IN EQUIP.idcentre%TYPE,
							p_descripcio  		IN EQUIP.descripcio%TYPE,
							p_dataCompra  		IN EQUIP.dataCompra%TYPE,
							p_dataFiGarantia	IN EQUIP.dataFiGarantia%TYPE,
							p_dataalta     		IN EQUIP.dataAlta%TYPE,
							p_databaixa    		IN EQUIP.dataBaixa%TYPE,
							RES             	OUT VARCHAR2);
													
	PROCEDURE PR_REUBICAR_EQUIP (p_idequip  	IN EQUIP.idequip%TYPE,
								p_idcentre  	IN EQUIP.idcentre%TYPE,
								RES             OUT VARCHAR2);
								
  	FUNCTION FN_OBTENIR_ID_EQUIP(p_numSerie EQUIP.numSerie%TYPE, p_idmodel EQUIP.idmodel%TYPE) 
	RETURN EQUIP.idequip%TYPE;								
								 
END;
/


CREATE OR REPLACE 
PACKAGE BODY   pfc_BDR.PKG_EQUIPS AS

  /**************************************
   *  MANTENIMENT CATEGORIES
   **************************************/
   	FUNCTION FN_OBTENIR_ID_NOM_CATEGORIA(p_nomCategoria CATEGORIA.nomCategoria%TYPE) 
	RETURN CATEGORIA.idcategoria%TYPE
	IS
		idAux CATEGORIA.idcategoria%TYPE; 
		q INTEGER;
	BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM CATEGORIA
	  WHERE p_nomCategoria = nomCategoria;

	  IF (q > 0) THEN
		  SELECT idcategoria 
			INTO idAux
		  FROM CATEGORIA
		  WHERE p_nomCategoria = nomCategoria;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
	END;
	
  PROCEDURE PR_ALTA_CATEGORIA (p_nomCategoria  IN CATEGORIA.nomCategoria%TYPE,
                               RES             OUT VARCHAR2)
  IS
	   E_CAMP_NUL         EXCEPTION;
	   E_CATEGORIA_EXISTEIX      EXCEPTION;

	   existeix      VARCHAR2(1);
	   param_entrada VARCHAR2(500);
	   err_num       NUMBER;
	   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NOMCATEGORIA: '||p_nomCategoria;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_nomCategoria) IS NULL)
     THEN
         RAISE E_CAMP_NUL ;
      END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE nomCategoria  = p_nomCategoria
			AND ROWNUM  = 1;

        RAISE E_CATEGORIA_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

      --Si ha passat les proves de validaci realitza la inserci
      INSERT INTO CATEGORIA (nomCategoria)
           VALUES (p_nomCategoria);

      RES := 'OK';
  
      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_ALTA_CATEGORIA', param_entrada, 'RES: '||RES);
      COMMIT;

  EXCEPTION
    WHEN E_CATEGORIA_EXISTEIX THEN
      RES := 'ERROR: la categoria ja ha estat donada d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_CATEGORIA', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_CATEGORIA', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_CATEGORIA', param_entrada, 'RES: '||RES);
      COMMIT;

  END;

  
  PROCEDURE PR_BAIXA_CATEGORIA (p_idcategoria    IN CATEGORIA.idcategoria%TYPE,
								RES          	 OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_CATEGORIA_NOEXISTEIX EXCEPTION;
   E_CATEGORIA_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

      param_entrada := 'P_idcategoria: '||p_idcategoria;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idcategoria) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que la categoria existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE idcategoria = p_idcategoria
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_CATEGORIA_NOEXISTEIX;
     END;

     --Verifica que la categoria no est donada de baixa
     BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM CATEGORIA
        WHERE idcategoria = p_idcategoria
          AND dataBaixa  IS NULL
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_CATEGORIA_JA_BAIXA;
      END;

      --Si ha passat les proves de validaci realitza l'esborrat lgic de la categoria
      UPDATE CATEGORIA SET dataBaixa = SYSDATE
      WHERE  idcategoria = p_idcategoria;

      RES := 'OK';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_BAIXA_CATEGORIA', param_entrada, 'RES: '||RES);
      COMMIT;

   EXCEPTION
      WHEN E_CATEGORIA_NOEXISTEIX THEN
        RES := 'ERROR: La categoria a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN E_CATEGORIA_JA_BAIXA THEN
        RES := 'ERROR: La categoria ja havia estat donada de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp ID CATEGORIA no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

  END;

  
  PROCEDURE PR_MODIF_CATEGORIA (p_idcategoria   IN CATEGORIA.idcategoria%TYPE,
								p_nomCategoria  IN CATEGORIA.nomCategoria%TYPE,
								p_dataalta     IN CATEGORIA.dataAlta%TYPE,
								p_databaixa    IN CATEGORIA.dataBaixa%TYPE,
								RES             OUT VARCHAR2)
  IS
   E_CAMP_NUL             EXCEPTION;
   E_CATEGORIA_NOEXISTEIX  EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDCATEGORIA: '||p_idcategoria||', P_NOMCATEGORIA: '||p_nomCategoria||', P_DATAALTA: '||p_dataAlta||', P_DATABAIXA: '||p_databaixa;

      --Verifica que tots els camps obligatoris tenen valor
      IF (TRIM(p_idcategoria) IS NULL) OR (TRIM(p_nomCategoria) IS NULL)
		THEN
			RAISE E_CAMP_NUL ;
      END IF;

      --Verifica que la categoria existeix en la Base de Dades
      BEGIN

       SELECT 'S'
         INTO EXISTEIX
         FROM CATEGORIA
        WHERE idcategoria = p_idcategoria
          AND ROWNUM   = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_CATEGORIA_NOEXISTEIX;
      END;

		-- si es modifica la data d'alta
       IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > p_databaixa) THEN
             RAISE E_DATA_BAIXA_INCORRECTA;
         END IF;
       END IF;
	   
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;	   
    
     --Si es passen les proves de validaci actualiza
     UPDATE CATEGORIA
        SET  nomCategoria = nvl(p_nomCategoria, nomCategoria),
             dataAlta    = nvl(p_dataalta, dataAlta),
             dataBaixa     = nvl(p_databaixa, dataBaixa)
      WHERE   idcategoria = p_idcategoria;
      commit;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION

      WHEN E_CATEGORIA_NOEXISTEIX THEN
        RES := 'ERROR: La categoria a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;
		
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;			

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_CATEGORIA', param_entrada, 'RES: '||RES);
        COMMIT;

  END;
  
  
  /**************************************
   *  MANTENIMENT MARQUES
   **************************************/
	FUNCTION FN_OBTENIR_ID_NOM_MARCA(p_nomMarca MARCA.nomMarca%TYPE) 
	RETURN MARCA.idmarca%TYPE
	IS
		idAux MARCA.idmarca%TYPE; 
		q INTEGER;
	BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM MARCA
	  WHERE p_nomMarca = nomMarca;

	  IF (q > 0) THEN
		  SELECT idmarca 
			INTO idAux
		  FROM MARCA
		  WHERE p_nomMarca = nomMarca;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
	END;
	
  PROCEDURE PR_ALTA_MARCA (p_nomMarca IN MARCA.nomMarca%TYPE,
							RES 	 	OUT VARCHAR2)
  IS
   E_CAMP_NUL         		EXCEPTION;
   E_MARCA_EXISTEIX     EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NOMMARCA: '||p_nomMarca;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_nomMarca) IS NULL) THEN
         RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE nomMarca  = p_nomMarca
			  AND ROWNUM  = 1;

        RAISE E_MARCA_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

     --Si ha passat les proves de validaci realitza la inserci
     INSERT INTO MARCA (nomMarca)
          VALUES (p_nomMarca);

     RES := 'OK';
  
     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_ALTA_MARCA', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
    WHEN E_MARCA_EXISTEIX THEN
      RES := 'ERROR: la marca ja ha estat donada d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MARCA', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MARCA', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MARCA', param_entrada, 'RES: '||RES);
      COMMIT;

  END;
							
							
  PROCEDURE PR_BAIXA_MARCA (p_idmarca IN MARCA.idmarca%TYPE,
							RES         OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_MARCA_NOEXISTEIX EXCEPTION;
   E_MARCA_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDMARCA: '||p_idmarca;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idmarca) IS NULL) 
		 THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que la categoria existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE idmarca = p_idmarca
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MARCA_NOEXISTEIX;
     END;

     --Verifica que la marca no est donada de baixa
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE idmarca = p_idmarca
			AND dataBaixa  IS NULL
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MARCA_JA_BAIXA;
     END;

     --Si ha passat les proves de validaci realitza l'esborrat lgic de la marca
     UPDATE MARCA SET dataBaixa = SYSDATE
     WHERE  idmarca = p_idmarca;

     RES := 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_BAIXA_MARCA', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION
      WHEN E_MARCA_NOEXISTEIX THEN
        RES := 'ERROR: La marca a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_MARCA_JA_BAIXA THEN
        RES := 'ERROR: La marca ja havia estat donada de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
  
							
  PROCEDURE PR_MODIF_MARCA (p_idmarca 	IN MARCA.idmarca%TYPE,
							p_nomMarca  IN MARCA.nomMarca%TYPE,
							p_dataalta  IN MARCA.dataAlta%TYPE,
							p_databaixa IN MARCA.dataBaixa%TYPE,
							RES         OUT VARCHAR2)
  IS
   E_CAMP_NUL             EXCEPTION;
   E_MARCA_NOEXISTEIX       EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDMARCA: '||p_idmarca||', P_NOMMARCA: '||p_nomMarca||', P_DATAALTA: '||p_dataalta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idmarca) IS NULL) OR (TRIM(p_nomMarca) IS NULL)
			THEN
				RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que la marca existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE idmarca = p_idmarca
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MARCA_NOEXISTEIX;
     END;

	 -- si es modifica la data d'alta
	 IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
		IF (p_dataalta > p_databaixa) THEN
				RAISE E_DATA_BAIXA_INCORRECTA;
		END IF;
	 END IF;
	 
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;
    
     --Si es passen les proves de validaci actualizem
     UPDATE MARCA
        SET  nomMarca = nvl(p_nomMarca, nomMarca),
             dataAlta    = nvl(p_dataalta, dataAlta),
             dataBaixa     = nvl(p_databaixa, dataBaixa)
     WHERE   idmarca = p_idmarca;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_MARCA_NOEXISTEIX THEN
        RES := 'ERROR: La marca a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;
		
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;			

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MARCA', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
								
								
  /**************************************
   *  MANTENIMENT MODELS
   **************************************/	
    FUNCTION FN_OBTENIR_ID_NOM_MODEL(p_idmarca MODEL.idmarca%TYPE, p_nomModel MODEL.nomModel%TYPE) 
	RETURN MODEL.idmodel%TYPE
	IS
		idAux MODEL.idmodel%TYPE; 
		q INTEGER;
	BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM MODEL
	  WHERE p_idmarca = idmarca
			AND p_nomModel = nomModel;

	  IF (q > 0) THEN
		  SELECT idmodel 
			INTO idAux
		  FROM MODEL
		  WHERE p_idmarca = idmarca
				AND p_nomModel = nomModel;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
	END;
  
	
  PROCEDURE PR_ALTA_MODEL (p_idmarca  			IN MODEL.idmarca%TYPE,
							p_nomModel  		IN MODEL.nomModel%TYPE,
							p_caracteristiques  IN MODEL.caracteristiques%TYPE,
							p_idcategoria  		IN MODEL.idcategoria%TYPE,
							RES 				OUT VARCHAR2)
  IS
   E_CAMP_NUL         	EXCEPTION;
   E_MODEL_EXISTEIX     EXCEPTION;
   E_MARCA_NOEXISTEIX   EXCEPTION;
   E_CATEGORIA_NOEXISTEIX   EXCEPTION;

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NOMMODEL: '||p_nomModel||', P_CARACTERISTIQUES: '||p_caracteristiques||', P_IDMARCA: '||p_idmarca||', P_IDCATEGORIA: '||p_idcategoria;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_nomModel) IS NULL) 
				OR (TRIM(p_idmarca) IS NULL) OR (TRIM(p_idcategoria) IS NULL)
		THEN 
         RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE nomModel  = p_nomModel
			  AND ROWNUM  = 1;

        RAISE E_MODEL_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
	 
	 --Verifica que la marca existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE idmarca  = p_idmarca
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_MARCA_NOEXISTEIX;
     END;
	 
	 --Verifica que la categoria existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE idcategoria  = p_idcategoria
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CATEGORIA_NOEXISTEIX;
     END;	 

     --Si ha passat les proves de validaci realitza la inserci
     INSERT INTO MODEL (idmarca,nomModel,caracteristiques,idcategoria)
          VALUES (p_idmarca,p_nomModel,p_caracteristiques,p_idcategoria);

     RES := 'OK';
  
     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
    WHEN E_MODEL_EXISTEIX THEN
      RES := 'ERROR: el model ja ha estat donat d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;
	  
	WHEN E_MARCA_NOEXISTEIX THEN
      RES := 'ERROR: la marca no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;	  

	WHEN E_CATEGORIA_NOEXISTEIX THEN
      RES := 'ERROR: la categoria no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;	  
	  
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;
  END;
							
							
  PROCEDURE PR_BAIXA_MODEL (p_idmodel IN MODEL.idmodel%TYPE,
							RES     	OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_MODEL_NOEXISTEIX EXCEPTION;
   E_MODEL_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

      param_entrada := 'P_IDMODEL: '||p_idmodel;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idmodel) IS NULL) 
		 THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el model existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE idmodel = p_idmodel
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MODEL_NOEXISTEIX;
     END;

     --Verifica que el model no est donat de baixa
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE idmodel = p_idmodel
			AND dataBaixa  IS NULL
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MODEL_JA_BAIXA;
     END;

     --Si ha passat les proves de validaci realitza l'esborrat lgic del model
     UPDATE MODEL SET dataBaixa = SYSDATE
     WHERE  idmodel = p_idmodel;

     RES := 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_BAIXA_MODEL', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_MODEL_NOEXISTEIX THEN
        RES := 'ERROR: El model a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_MODEL_JA_BAIXA THEN
        RES := 'ERROR: El model ja havia estat donat de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
							
							
  PROCEDURE PR_MODIF_MODEL (p_idmodel 		IN MODEL.idmodel%TYPE,
							p_nomModel  		IN MODEL.nomModel%TYPE,
							p_caracteristiques  IN MODEL.caracteristiques%TYPE,
							p_idmarca  			IN MODEL.idmarca%TYPE,
							p_idcategoria  		IN MODEL.idcategoria%TYPE,				
							p_dataalta  		IN MODEL.dataAlta%TYPE,
							p_databaixa 		IN MODEL.dataBaixa%TYPE,
							RES     			OUT VARCHAR2)
  IS
   E_CAMP_NUL             EXCEPTION;
   E_MODEL_NOEXISTEIX       EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_MARCA_NOEXISTEIX   EXCEPTION;
   E_CATEGORIA_NOEXISTEIX   EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDMODEL: '||p_idmodel||', P_NOMMODEL: '||p_nomModel||', P_CARACTERISTIQUES: '||p_caracteristiques||', P_IDMARCA: '||p_idmarca||', P_IDCATEGORIA: '||p_idcategoria||', P_DATAALTA: '||p_dataalta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idmodel) IS NULL) OR (TRIM(p_nomModel) IS NULL) OR (TRIM(p_caracteristiques) IS NULL)
			OR (TRIM(p_idmarca) IS NULL) OR (TRIM(p_idcategoria) IS NULL)
				THEN
					RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el model existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE idmodel = p_idmodel
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_MODEL_NOEXISTEIX;
     END;

	 -- si es modifica la data d'alta
	 IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
		IF (p_dataalta > p_databaixa) THEN
				RAISE E_DATA_BAIXA_INCORRECTA;
		END IF;
	 END IF;
	 
	 --Verifica que la marca existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MARCA
        WHERE idmarca  = p_idmarca
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_MARCA_NOEXISTEIX;
     END;
	 
	 --Verifica que la categoria existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE idcategoria  = p_idcategoria
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CATEGORIA_NOEXISTEIX;
     END;	
	 
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;	 
    
     --Si es passen les proves de validaci actualizem
     UPDATE MODEL
        SET nomModel = nvl(p_nomModel, nomModel),
			caracteristiques = nvl(p_caracteristiques, caracteristiques),
			idmarca = nvl(p_idmarca, idmarca),
			idcategoria = nvl(p_idcategoria, idcategoria),
            dataAlta    = nvl(p_dataalta, dataAlta),
            dataBaixa   = nvl(p_databaixa, dataBaixa)
     WHERE   idmodel = p_idmodel;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_MODEL_NOEXISTEIX THEN
        RES := 'ERROR: El model a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_DATA_BAIXA_INCORRECTA THEN
        RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;
		
	WHEN E_MARCA_NOEXISTEIX THEN
      RES := 'ERROR: la marca no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;	  

	WHEN E_CATEGORIA_NOEXISTEIX THEN
      RES := 'ERROR: la categoria no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
      COMMIT;	  		

      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;		  
	  
     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
        COMMIT;
  END;

     
  /**************************************
   *  MANTENIMENT EQUIPS
   **************************************/   
    FUNCTION FN_OBTENIR_ID_EQUIP(p_numSerie EQUIP.numSerie%TYPE, p_idmodel EQUIP.idmodel%TYPE) 
	RETURN EQUIP.idequip%TYPE
	IS
		idAux EQUIP.idequip%TYPE; 
		q INTEGER;
	BEGIN
	  SELECT count(*) as d  
	  INTO q
	  FROM EQUIP
	  WHERE p_numSerie = numSerie
			AND p_idmodel = idmodel;

	  IF (q > 0) THEN
		  SELECT idequip 
			INTO idAux
		  FROM EQUIP
		  WHERE p_numSerie = numSerie
				AND p_idmodel = idmodel;
		  RETURN idAux;
	  ELSE 
		  RETURN 0;  --si no existeix retorna 0
	  END IF;
	END;
	
	
  PROCEDURE PR_ALTA_EQUIP (p_numSerie 		IN EQUIP.numSerie%TYPE,
							p_idmodel 			IN EQUIP.idmodel%TYPE,
							p_idcategoria  		IN EQUIP.idcategoria%TYPE,
							p_idcentre  		IN EQUIP.idcentre%TYPE,
							p_descripcio  		IN EQUIP.descripcio%TYPE,
							p_dataCompra  		IN EQUIP.dataCompra%TYPE,
							p_dataFiGarantia	IN EQUIP.dataFiGarantia%TYPE,
							RES 				OUT VARCHAR2)
  IS
   E_CAMP_NUL         	EXCEPTION;
   E_EQUIP_EXISTEIX     EXCEPTION;
   E_MODEL_NOEXISTEIX   EXCEPTION;
   E_CATEGORIA_NOEXISTEIX   EXCEPTION;
   E_CENTRE_NOEXISTEIX   EXCEPTION;
   E_CATEGORIA_INCORRECTE EXCEPTION;
   E_DATACOMPRA_IMPOSSIBLE EXCEPTION;
   E_PERIODE_INCORRECTE EXCEPTION;
   

   existeix      VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_NUMSERIE: '||p_numSerie||', P_IDMODEL: '||p_idmodel||', P_IDCATEGORIA: '||p_idcategoria||', P_IDCENTRE: '||p_idcentre||', P_DESCRIPCIO: '||p_descripcio||', P_DATACOMPRA: '||p_dataCompra||', P_DATAFIGARANTIA: '||p_dataFiGarantia;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_numSerie) IS NULL) OR (TRIM(p_idmodel) IS NULL) OR (TRIM(p_idcategoria) IS NULL) 
			OR (TRIM(p_idcentre) IS NULL) OR (TRIM(p_descripcio) IS NULL) OR (TRIM(p_dataCompra) IS NULL)
			 OR (TRIM(p_dataFiGarantia) IS NULL)
		THEN 
         RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que el registre no s'hagi inserit previament
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM EQUIP
        WHERE numSerie  = p_numSerie
			  AND idmodel  = p_idmodel
			  AND ROWNUM  = 1;

        RAISE E_EQUIP_EXISTEIX;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
	 
	 --Verifica que el model existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE idmodel  = p_idmodel
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_MODEL_NOEXISTEIX;
     END;
	 
	 --Verifica que la categoria existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE idcategoria  = p_idcategoria
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CATEGORIA_NOEXISTEIX;
     END;	

	 --Verifica que el centre existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CENTRE
        WHERE idcentre  = p_idcentre
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CENTRE_NOEXISTEIX;
     END;	 
	 
	 --Verifica que la categoria de l'equip s la mateixa que la categoria del model de l'equip
	 BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM DUAL
        WHERE p_idcategoria  = (SELECT idcategoria
								FROM MODEL
								WHERE idmodel = p_idmodel
										AND ROWNUM  = 1);
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CATEGORIA_INCORRECTE;
     END;
	 
	   --verifica que le data de compra s anterior a la data actual
         IF (p_dataCompra > SYSDATE) THEN
             RAISE E_DATACOMPRA_IMPOSSIBLE;
         END IF; 
		 
	   --verifica que la data de compra ha de ser inferior o igual a la data de fi de garantia
         IF (p_dataCompra > p_dataFiGarantia)  THEN
             RAISE E_PERIODE_INCORRECTE;
         END IF;		 

     --Si ha passat les proves de validaci realitza la inserci
     INSERT INTO EQUIP (numSerie,idmodel,idcategoria,idcentre,descripcio,dataCompra,dataFiGarantia)
          VALUES (p_numSerie,p_idmodel,p_idcategoria,p_idcentre,p_descripcio,p_dataCompra,p_dataFiGarantia);

     RES := 'OK';
  
     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
    WHEN E_EQUIP_EXISTEIX THEN
      RES := 'ERROR: l equip ja ha estat donat d alta';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;

    WHEN E_CAMP_NUL  THEN
      RES := 'ERROR: El camp no pot ser nul';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;
	  
	WHEN E_MODEL_NOEXISTEIX THEN
      RES := 'ERROR: el model no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;	  

	WHEN E_CATEGORIA_NOEXISTEIX THEN
      RES := 'ERROR: la categoria no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;	  
	  
	WHEN E_CENTRE_NOEXISTEIX THEN
      RES := 'ERROR: el centre no existeix en la base de dades';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;	

	WHEN E_CATEGORIA_INCORRECTE THEN
      RES := 'ERROR: la categoria de l equip ha de ser la mateixa que la categoria del model de l equip';

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;	  
	  
      WHEN E_DATACOMPRA_IMPOSSIBLE THEN
        RES := 'ERROR: La data de compra no pot ser posterior a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;		  
		
      WHEN E_PERIODE_INCORRECTE THEN
        RES := 'ERROR: la data de compra ha de ser inferior o igual a la data de fi de garantia';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;	 		

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'PKG_EQUIPS.PR_ALTA_EQUIP', param_entrada, 'RES: '||RES);
      COMMIT;
  END;
							
							
  PROCEDURE PR_BAIXA_EQUIP (p_idequip IN EQUIP.idequip%TYPE,
							RES         OUT VARCHAR2)
  IS
   E_CAMP_NUL  EXCEPTION;
   E_EQUIP_NOEXISTEIX EXCEPTION;
   E_EQUIP_JA_BAIXA EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

      param_entrada := 'P_IDEQUIP: '||p_idequip;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idequip) IS NULL) 
		 THEN
			RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l equip existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM EQUIP
        WHERE idequip = p_idequip
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_EQUIP_NOEXISTEIX;
     END;

     --Verifica que l equip no est donat de baixa
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM EQUIP
        WHERE idequip = p_idequip
			AND dataBaixa  IS NULL
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_EQUIP_JA_BAIXA;
     END;

     --Si ha passat les proves de validaci realitza l'esborrat lgic de l equip
     UPDATE EQUIP SET dataBaixa = SYSDATE
     WHERE  idequip = p_idequip;

     RES := 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
          VALUES(SYSDATE,'PKG_EQUIPS.PR_BAIXA_EQUIP', param_entrada, 'RES: '||RES);
     COMMIT;

  EXCEPTION
      WHEN E_EQUIP_NOEXISTEIX THEN
        RES := 'ERROR: L equip a donar de baixa no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_EQUIP_JA_BAIXA THEN
        RES := 'ERROR: L equip ja havia estat donat de baixa';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_BAIXA_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
  
							
  PROCEDURE PR_MODIF_EQUIP (p_idequip   		IN EQUIP.idequip%TYPE,
							p_numSerie 			IN EQUIP.numSerie%TYPE,
							p_idmodel 			IN EQUIP.idmodel%TYPE,
							p_idcategoria  		IN EQUIP.idcategoria%TYPE,
							p_idcentre  		IN EQUIP.idcentre%TYPE,
							p_descripcio  		IN EQUIP.descripcio%TYPE,
							p_dataCompra  		IN EQUIP.dataCompra%TYPE,
							p_dataFiGarantia	IN EQUIP.dataFiGarantia%TYPE,
							p_dataalta     		IN EQUIP.dataAlta%TYPE,
							p_databaixa    		IN EQUIP.dataBaixa%TYPE,
							RES             	OUT VARCHAR2)
  IS
   E_CAMP_NUL             EXCEPTION;
   E_EQUIP_NOEXISTEIX       EXCEPTION;
   E_DATA_BAIXA_INCORRECTA EXCEPTION;
   E_MODEL_NOEXISTEIX   EXCEPTION;
   E_CATEGORIA_NOEXISTEIX   EXCEPTION;
   E_CENTRE_NOEXISTEIX   EXCEPTION;
   E_DATACOMPRA_IMPOSSIBLE EXCEPTION;
   E_PERIODE_INCORRECTE EXCEPTION;
   E_DATA_IMPOSSIBLE EXCEPTION;

   vdcompra EQUIP.dataCompra%TYPE;
   vdfig EQUIP.dataFiGarantia%TYPE;
   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

   BEGIN

     param_entrada := 'P_IDEQUIP: '||p_idequip||', P_NUMSERIE: '||p_numSerie||', P_IDMODEL: '||p_idmodel||', P_IDCATEGORIA: '||p_idcategoria||', P_IDCENTRE: '||p_idcentre||', P_DESCRIPCIO: '||p_descripcio||', P_DATACOMPRA: '||p_dataCompra||', P_DATAFIGARANTIA: '||p_dataFiGarantia||', P_DATAALTA: '||p_dataalta||', P_DATABAIXA: '||p_databaixa;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idequip) IS NULL) OR (TRIM(p_numSerie) IS NULL) OR (TRIM(p_idmodel) IS NULL)
			OR (TRIM(p_idcategoria) IS NULL) OR (TRIM(p_idcentre) IS NULL) OR (TRIM(p_descripcio) IS NULL)
				OR (TRIM(p_dataCompra) IS NULL) OR (TRIM(p_dataFiGarantia) IS NULL) 
					THEN
						RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l equip existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM EQUIP
        WHERE idequip = p_idequip
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_EQUIP_NOEXISTEIX;
     END;

	 -- si es modifica la data d'alta
	 IF (p_dataalta IS NOT NULL AND p_databaixa IS NOT NULL) THEN
		IF (p_dataalta > p_databaixa) THEN
				RAISE E_DATA_BAIXA_INCORRECTA;
		END IF;
	 END IF;
	 
	 	 --Verifica que el model existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM MODEL
        WHERE idmodel  = p_idmodel
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_MODEL_NOEXISTEIX;
     END;
	 
	 --Verifica que la categoria existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CATEGORIA
        WHERE idcategoria  = p_idcategoria
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CATEGORIA_NOEXISTEIX;
     END;	

	 --Verifica que el centre existeixi
     BEGIN
        SELECT 'S'
        INTO EXISTEIX
        FROM CENTRE
        WHERE idcentre  = p_idcentre
			  AND ROWNUM  = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
			RAISE E_CENTRE_NOEXISTEIX;
     END;	 
	 
	   --verifica que la data de compra s anterior a la data actual
	   IF (p_dataCompra IS NOT NULL) THEN	   
         IF (p_dataCompra > SYSDATE) THEN
             RAISE E_DATACOMPRA_IMPOSSIBLE;
         END IF;
		END IF;
		
 	--verifica que la data de compra ha de ser inferior o igual a la data de fi de garantia
	IF (p_dataCompra IS NOT NULL) OR (p_dataFiGarantia IS NOT NULL) THEN
			
	   SELECT dataCompra, dataFiGarantia
	   INTO vdcompra, vdfig
	   FROM EQUIP
	   WHERE idequip = p_idequip;
	    
		IF (p_dataCompra IS NOT NULL) AND (p_dataFiGarantia IS NULL) THEN
          IF (p_dataCompra > vdfig)  THEN
                 RAISE E_PERIODE_INCORRECTE;
          END IF;		 
	    END IF;
		
		IF (p_dataCompra IS NOT NULL) AND (p_dataFiGarantia IS NOT NULL) THEN
          IF (p_dataCompra > p_dataFiGarantia)  THEN
                 RAISE E_PERIODE_INCORRECTE;
          END IF;		 
	    END IF;
		
		IF (p_dataCompra IS NULL) AND (p_dataFiGarantia IS NOT NULL) THEN
          IF (vdcompra > p_dataFiGarantia)  THEN
                 RAISE E_PERIODE_INCORRECTE;
          END IF;		 
	    END IF;		
		
	END IF;	
	
	   --verifica que les dates sn anteriors a la data actual
       IF (p_dataalta IS NOT NULL OR p_databaixa IS NOT NULL) THEN
         IF (p_dataalta > SYSDATE) OR (p_databaixa > SYSDATE) THEN
             RAISE E_DATA_IMPOSSIBLE;
         END IF;
       END IF;	
    
     --Si es passen les proves de validaci actualizem
     UPDATE EQUIP
        SET numSerie = nvl(p_numSerie, numSerie),
			idmodel = nvl(p_idmodel, idmodel),
			idcategoria = nvl(p_idcategoria, idcategoria),
			idcentre = nvl(p_idcentre, idcentre),
			descripcio = nvl(p_descripcio, descripcio),
			dataCompra = nvl(p_dataCompra, dataCompra),
			dataFiGarantia = nvl(p_dataFiGarantia, dataFiGarantia),
            dataAlta    = nvl(p_dataalta, dataAlta),
            dataBaixa   = nvl(p_databaixa, dataBaixa)
     WHERE   idequip = p_idequip;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION
      WHEN E_EQUIP_NOEXISTEIX THEN
        RES := 'ERROR: L equip a modificar no existeix';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;
		
		WHEN E_MODEL_NOEXISTEIX THEN
		  RES := 'ERROR: el model no existeix en la base de dades';

		  INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
			 VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
		  COMMIT;	  

		WHEN E_CATEGORIA_NOEXISTEIX THEN
		  RES := 'ERROR: la categoria no existeix en la base de dades';

		  INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
			 VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
		  COMMIT;	  
		  
		WHEN E_CENTRE_NOEXISTEIX THEN
		  RES := 'ERROR: el centre no existeix en la base de dades';

		  INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
			 VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_MODEL', param_entrada, 'RES: '||RES);
		  COMMIT;			

	  WHEN E_DATA_BAIXA_INCORRECTA THEN
		RES := 'ERROR: La data d alta no pot ser posterior a la data de baixa';

		INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
		   VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
		COMMIT;
		
      WHEN E_DATACOMPRA_IMPOSSIBLE THEN
        RES := 'ERROR: La data de compra no pot ser posterior a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;		  
		
      WHEN E_PERIODE_INCORRECTE THEN
        RES := 'ERROR: la data de compra ha de ser inferior o igual a la data de fi de garantia';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;	 		
		
      WHEN E_DATA_IMPOSSIBLE THEN
        RES := 'ERROR: La data d alta o la data de baixa no poden ser posteriors a la data actual';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;			

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_MODIF_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
						
							
  PROCEDURE PR_REUBICAR_EQUIP (p_idequip  	IN EQUIP.idequip%TYPE,
							p_idcentre  	IN EQUIP.idcentre%TYPE,
							RES             OUT VARCHAR2)							
  IS
   E_CAMP_NUL             	EXCEPTION;
   E_EQUIP_NOEXISTEIX       EXCEPTION;
   E_CENTRE_NOEXISTEIX 	EXCEPTION;

   existeix        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);

  BEGIN

     param_entrada := 'P_IDEQUIP: '||p_idequip||', P_IDCENTRE: '||p_idcentre;

     --Verifica que tots els camps obligatoris tenen valor
     IF (TRIM(p_idequip) IS NULL) OR (TRIM(p_idcentre) IS NULL) 
			THEN
				RAISE E_CAMP_NUL ;
     END IF;

     --Verifica que l equip existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM EQUIP
        WHERE idequip = p_idequip
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_EQUIP_NOEXISTEIX;
     END;
	 
	 --Verifica que el centre existeix en la Base de Dades
     BEGIN
		SELECT 'S'
        INTO EXISTEIX
        FROM CENTRE
        WHERE idcentre = p_idcentre
			AND ROWNUM   = 1;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_CENTRE_NOEXISTEIX;
     END;
 
     --Si es passen les proves de validaci actualizem
     UPDATE EQUIP
        SET idcentre = nvl(p_idcentre, idcentre)
     WHERE   idequip = p_idequip;
     COMMIT;

     RES:= 'OK';

     INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'PKG_EQUIPS.PR_REUBICAR_EQUIP', param_entrada, 'RES: '||RES);
     COMMIT;

   EXCEPTION
      WHEN E_EQUIP_NOEXISTEIX THEN
        RES := 'ERROR: L equip no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_REUBICAR_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;
		
      WHEN E_CENTRE_NOEXISTEIX THEN
        RES := 'ERROR: El centre no existeix en la base de dades';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_REUBICAR_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;		

      WHEN E_CAMP_NUL  THEN
        RES := 'ERROR: El camp no pot ser nul';

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_REUBICAR_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RES:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (dataLog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE, 'PKG_EQUIPS.PR_REUBICAR_EQUIP', param_entrada, 'RES: '||RES);
        COMMIT;
  END;
                                                                 
END;
/

COMMIT;

spool off
exit