-----------------------------------------
-----------------------------------------
spool C:\UOC\PFC\mbernard\logs\ap6-package_llistats.log
-----------------------------------------
-----------------------------------------


ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS';
ALTER SESSION SET NLS_TIMESTAMP_FORMAT = 'DD/MM/YYYY HH24:MI:SS';
ALTER SESSION SET NLS_TIME_FORMAT = 'HH24:MI:SS';
COMMIT;



CREATE OR REPLACE PACKAGE pfc_BDR.PKG_LLISTATS AS
/******************************************************************************
   NOM:   PKG_LLISTATS
   OBJECTIU: LLISTATS DE LES DIFERENTS TAULES
******************************************************************************/

  /**************************************
  *  LLISTATS CATEGORIES
  **************************************/
  TYPE CATEGORIES_REC IS RECORD (
    idcategoria CATEGORIA.idcategoria%TYPE 
    , nomCategoria CATEGORIA.nomCategoria%TYPE 
  );
  
  TYPE CATEGORIES_TAB is table of CATEGORIES_REC;  
   
  FUNCTION FN_LLISTA_CATEGORIES
  RETURN CATEGORIES_TAB PIPELINED; 
  --retorna un llistat de totes les categories
  
  /**************************************
  *  LLISTATS CENTRES
  **************************************/
  TYPE CENTRES_REC IS RECORD (
    idcentre CENTRE.idcentre%TYPE 
    , nomCentre CENTRE.nomCentre%TYPE 
	, adrea CENTRE.adrea%TYPE 
	, poblacio VARCHAR2(50) 
	, codipostal CENTRE.codipostal%TYPE 
	, telefon CENTRE.telefon%TYPE 
	, mail CENTRE.mail%TYPE 
  );
  
  TYPE CENTRES_TAB is table of CENTRES_REC;  
   
  FUNCTION FN_LLISTA_CENTRES
  RETURN CENTRES_TAB PIPELINED;   
  --retorna un llistat de tots els centres
  
  
  /**************************************
  *  LLISTATS EQUIPS 
  **************************************/
  TYPE EQUIPS_CENTRE_REC IS RECORD (
    idcentre CENTRE.idcentre%TYPE 
    , nomCentre CENTRE.nomCentre%TYPE 
	, nomCategoria CATEGORIA.nomCategoria%TYPE 
	, nomMarca MARCA.nomMarca%TYPE 
	, nomModel MODEL.nomModel%TYPE 
	, idequip EQUIP.idequip%TYPE 
	, numserie EQUIP.numserie%TYPE
	, descripcio EQUIP.descripcio%TYPE
	, datacompra EQUIP.datacompra%TYPE
	, datafigarantia EQUIP.datafigarantia%TYPE
  );

  TYPE EQUIPS_CENTRE_TAB is table of EQUIPS_CENTRE_REC;  
  
  TYPE MANTENIMENTS_EQUIPS_REC IS RECORD (
	nomCategoria CATEGORIA.nomCategoria%TYPE
	, idequip EQUIP.idequip%TYPE 
	, numserie EQUIP.numserie%TYPE
 	, nomMarca MARCA.nomMarca%TYPE 
	, nomModel MODEL.nomModel%TYPE  
	, descripcio EQUIP.descripcio%TYPE
	, datacompra EQUIP.datacompra%TYPE	
    , nomCentre CENTRE.nomCentre%TYPE 
	, tipusManteniment VARCHAR2(20)
    , datainici DATE
    , datafi DATE
    , nomempresa EMPRESA.nomEmpresa%TYPE
    , observacions VARCHAR2(100)
  );

  TYPE MANTENIMENTS_EQUIPS_TAB is table of MANTENIMENTS_EQUIPS_REC; 
  
  TYPE REVISIONS_PENDENTS_REC IS RECORD (
	dataPrevista DATE
	, observacio VARCHAR2(100)
	, nomCategoria CATEGORIA.nomCategoria%TYPE
	, idequip EQUIP.idequip%TYPE 
	, numserie EQUIP.numserie%TYPE
 	, nomMarca MARCA.nomMarca%TYPE 
	, nomModel MODEL.nomModel%TYPE  
	, descripcio EQUIP.descripcio%TYPE
	, datacompra EQUIP.datacompra%TYPE	
    , nomCentre CENTRE.nomCentre%TYPE 
  );
  
  TYPE REVISIONS_PENDENTS_TAB is table of REVISIONS_PENDENTS_REC;   

  TYPE MANTENIMENTS_EMPRESA_REC IS RECORD (
	idempresa EMPRESA.idEmpresa%TYPE 
    , nomempresa EMPRESA.nomEmpresa%TYPE  
    , nif EMPRESA.nif%TYPE 
    , adrea EMPRESA.adrea%TYPE 
	, nompoblacio POBLACIO.nomPoblacio%TYPE 
    , telefon1 EMPRESA.telefon1%TYPE 
    , fax EMPRESA.fax%TYPE 
    , mail EMPRESA.mail%TYPE 
    , personaContacte EMPRESA.personaContacte%TYPE 	
    , telefonContacte EMPRESA.telefonContacte%TYPE 
    , telefon24hores EMPRESA.telefon24hores%TYPE 
    , datainici DATE
    , datafi DATE	
	, tipusManteniment VARCHAR2(20)	
	, observacions VARCHAR2(100)
	, nomCategoria CATEGORIA.nomCategoria%TYPE
	, idequip EQUIP.idequip%TYPE 
	, numserie EQUIP.numserie%TYPE
 	, nomMarca MARCA.nomMarca%TYPE 
	, nomModel MODEL.nomModel%TYPE  
	, descripcio EQUIP.descripcio%TYPE
	, datacompra EQUIP.datacompra%TYPE	
    , nomCentre CENTRE.nomCentre%TYPE 
  );

  TYPE MANTENIMENTS_EMPRESA_TAB is table of MANTENIMENTS_EMPRESA_REC;   
  
     
  FUNCTION FN_LLISTA_EQUIPS_CENTRE(vidcentre CENTRE.idcentre%TYPE)
  RETURN EQUIPS_CENTRE_TAB PIPELINED;   
  --retorna un llistat de tots els equips del centre indicat
  --si el centre no existeix, retorna tots els equips de tots els centres
  --ordenat per centre i per categoria
  
  FUNCTION FN_LLISTA_MANTENIMENTS_EQUIPS
  RETURN MANTENIMENTS_EQUIPS_TAB PIPELINED;   
  --retorna un llistat de tots els manteniments dels equips 
  --ordenat per categoria, equip i data del manteniment
  
  FUNCTION FN_LLISTA_REVISIONS_PENDENTS
  RETURN REVISIONS_PENDENTS_TAB PIPELINED;   
  --retorna un llistat de les revisions pendents dels equips 
  --ordenat per data prevista de la revisi
  
  FUNCTION FN_LLISTA_MANTENIMENTS_EMPRESA
  RETURN MANTENIMENTS_EMPRESA_TAB PIPELINED;   
  --retorna un llistat dels manteniments realitzats per les empreses de servei 
  --ordenat per empresa i data del manteniment  
  					
END;
/


CREATE OR REPLACE 
PACKAGE BODY pfc_BDR.PKG_LLISTATS AS

	FUNCTION FN_LLISTA_CATEGORIES 
	RETURN CATEGORIES_TAB PIPELINED
	IS 
	  CURSOR cCategories IS
		SELECT idcategoria, nomCategoria
		FROM pfc_BDR.CATEGORIA
		WHERE dataBaixa IS NULL
		;
	BEGIN          
	  FOR regCateg IN cCategories LOOP
	    PIPE ROW(regCateg);
	  END LOOP;
  
      RETURN;
	END;
	
	
	FUNCTION FN_LLISTA_CENTRES 
	RETURN CENTRES_TAB PIPELINED
	IS 
	  CURSOR cCentres IS
		SELECT c.idcentre, c.nomCentre, c.adrea
               , p.nomPoblacio
               , c.codipostal, c.telefon, c.mail
		FROM pfc_BDR.CENTRE c
             , pfc_BDR.POBLACIO p
		WHERE c.dataBaixa IS NULL
              AND c.idpoblacio = p.idpoblacio
		;
	BEGIN          
	  FOR regCentre IN cCentres LOOP
	    PIPE ROW(regCentre);
	  END LOOP;
  
      RETURN;
	END;
	
	
	FUNCTION FN_LLISTA_EQUIPS_CENTRE(vidcentre CENTRE.idcentre%TYPE) 
	RETURN EQUIPS_CENTRE_TAB PIPELINED
	IS 
	  CURSOR cEquipsCentres IS
			SELECT 
					c.idcentre, c.nomCentre
					, ca.nomcategoria
					, ma.nommarca
					, m.nommodel
					, e.idequip, e.numserie, e.descripcio
					, e.datacompra, e.datafigarantia
			FROM pfc_BDR.CENTRE c
				 , pfc_BDR.EQUIP e
				 , pfc_BDR.CATEGORIA ca
				 , pfc_BDR.MODEL m
				 , pfc_BDR.MARCA ma
			WHERE c.dataBaixa IS NULL
				  AND c.idcentre = e.idcentre
				  AND e.dataBaixa IS NULL
				  AND e.idcategoria = ca.idcategoria
				  AND e.idmodel = m.idmodel
				  AND m.idmarca = ma.idmarca   
			ORDER BY c.idcentre, ca.nomcategoria, ma.nommarca, m.nommodel, e.datacompra
			;
	  CURSOR cEquips1Centre IS
			SELECT 
					c.idcentre, c.nomCentre
					, ca.nomcategoria
					, ma.nommarca
					, m.nommodel
					, e.idequip, e.numserie, e.descripcio
					, e.datacompra, e.datafigarantia
			FROM pfc_BDR.CENTRE c
				 , pfc_BDR.EQUIP e
				 , pfc_BDR.CATEGORIA ca
				 , pfc_BDR.MODEL m
				 , pfc_BDR.MARCA ma
			WHERE c.dataBaixa IS NULL
				  AND c.idcentre = vidcentre
				  AND c.idcentre = e.idcentre
				  AND e.dataBaixa IS NULL
				  AND e.idcategoria = ca.idcategoria
				  AND e.idmodel = m.idmodel
				  AND m.idmarca = ma.idmarca   
			ORDER BY c.idcentre, ca.nomcategoria, ma.nommarca, m.nommodel, e.datacompra
			;
	  qcentres integer;
	BEGIN    
	  SELECT COUNT(*)
	    INTO qcentres
	  FROM CENTRE
	  WHERE idcentre = vidcentre;	  
	  
	  IF (qcentres > 0) THEN 	
		  FOR regEquips1Centre IN cEquips1Centre LOOP
			PIPE ROW(regEquips1Centre);
		  END LOOP;
	  ELSE 
		  FOR regEquipsCentres IN cEquipsCentres LOOP
			PIPE ROW(regEquipsCentres);
		  END LOOP;	  
	  END IF;
	  
      RETURN;
	END;	


	FUNCTION FN_LLISTA_MANTENIMENTS_EQUIPS 
	RETURN MANTENIMENTS_EQUIPS_TAB PIPELINED
	IS 
	  CURSOR cManteniments IS
			SELECT 
				  ca.nomcategoria
				  , m.idequip
				  , e.numserie
				  , ma.nommarca marca
				  , mo.nommodel model
				  , e.descripcio
				  , e.datacompra
				  , ce.nomcentre centre
				  , m.tipusManteniment
				  , m.datainici
				  , m.datafi
				  , m.nomempresa empresa
				  , m.observacions
			FROM 
				  (select 
						 mc.idequip 
						 , 'MANT. CORRECTIU' tipusManteniment
						 , mc.idmanteniment
						 , mc.datainici
						 , mc.datafi
						 , mc.idempresa
						 , em.nomempresa
						 , mc.observacions
				  from pfc_BDR.resoluciomant mc
					   , pfc_BDR.empresa em
				  where mc.dataBaixa is null
						AND mc.idempresa = em.idempresa
				  UNION ALL
				  select 
						 mp.idequip
						 , 'MANT. PREVENTIU' tipusManteniment
						 , mp.idmanteniment
						 , mp.datarealitzacio datainici
						 , mp.datarealitzacio datafi
						 , mp.idempresa
						 , em.nomempresa
						 , mp.observacions
				  from pfc_BDR.revisio mp
					   , pfc_BDR.empresa em
				  where mp.dataBaixa is null
						AND mp.idempresa = em.idempresa --si empresa emplenat => revisio realitzada
				  ) M
				, pfc_BDR.equip e
				, pfc_BDR.categoria ca
				, pfc_BDR.model mo
				, pfc_BDR.marca ma
				, pfc_BDR.centre ce
			WHERE m.idequip = e.idequip
				  AND e.dataBaixa is null
				  AND e.idcategoria = ca.idcategoria
				  AND e.idmodel = mo.idmodel
				  AND mo.idmarca = ma.idmarca
				  AND e.idcentre = ce.idcentre
			ORDER BY ca.nomcategoria, m.idequip, m.datainici    
			;
	BEGIN          
	  FOR regMants IN cManteniments LOOP
	    PIPE ROW(regMants);
	  END LOOP;
  
      RETURN;
	END;
	
	
	FUNCTION FN_LLISTA_REVISIONS_PENDENTS 
	RETURN REVISIONS_PENDENTS_TAB PIPELINED
	IS 
	  CURSOR cRevisions IS
			SELECT 
				   mp.dataPrevista
				   , DECODE(sign(mp.dataPrevista - sysdate), -1
		,'ALARMA!!! Fa '|| trunc(sysdate - mp.dataPrevista) ||' dies que s''hauria d''haver realitzat la revisi'
		, 'OK. Falten '|| trunc(mp.dataPrevista - sysdate) ||' dies per la segent revisi') observacio
				   , ca.nomcategoria
				   , mp.idequip
				   , e.numserie
				   , ma.nommarca marca
				   , mo.nommodel model
				   , e.descripcio
				   , e.datacompra
				   , ce.nomcentre centre
			FROM pfc_BDR.revisio mp
				 , pfc_BDR.equip e
				 , pfc_BDR.categoria ca
				 , pfc_BDR.model mo
				 , pfc_BDR.marca ma
				 , pfc_BDR.centre ce
			WHERE mp.dataBaixa is null
				  AND mp.datarealitzacio IS NULL
				  AND mp.idequip = e.idequip
				  AND e.dataBaixa is null
				  AND e.idcategoria = ca.idcategoria
				  AND e.idmodel = mo.idmodel
				  AND mo.idmarca = ma.idmarca
				  AND e.idcentre = ce.idcentre
			ORDER BY mp.dataPrevista			
			;
	BEGIN          
	  FOR regRevs IN cRevisions LOOP
	    PIPE ROW(regRevs);
	  END LOOP;
  
      RETURN;
	END;
	
	
	FUNCTION FN_LLISTA_MANTENIMENTS_EMPRESA 
	RETURN MANTENIMENTS_EMPRESA_TAB PIPELINED
	IS 
	  CURSOR cManteniments IS
			SELECT 
				  m.idempresa
				  , em.nomempresa empresa
				  , em.nif
				  , em.adrea
				  , p.nompoblacio
				  , em.telefon1
				  , em.fax
				  , em.mail
				  , em.personaContacte
				  , em.telefonContacte
				  , em.telefon24hores
				  , m.datainici
				  , m.datafi
				  , m.tipusManteniment          
				  , m.observacions  
				  , ca.nomcategoria
				  , m.idequip
				  , e.numserie
				  , ma.nommarca marca
				  , mo.nommodel model
				  , e.descripcio
				  , e.datacompra
				  , ce.nomcentre centre
			FROM 
					  (select 
							 mc.idequip 
							 , 'MANT. CORRECTIU' tipusManteniment
							 , mc.idmanteniment
							 , mc.datainici
							 , mc.datafi
							 , mc.idempresa
							 , mc.observacions
					  from pfc_BDR.resoluciomant mc
					  where mc.dataBaixa is null
					  UNION ALL
					  select 
							 mp.idequip
							 , 'MANT. PREVENTIU' tipusManteniment
							 , mp.idmanteniment
							 , mp.datarealitzacio datainici
							 , mp.datarealitzacio datafi
							 , mp.idempresa
							 , mp.observacions
					  from pfc_BDR.revisio mp
					  where mp.dataBaixa is null
					  ) M
					, pfc_BDR.equip e
					, pfc_BDR.categoria ca
					, pfc_BDR.model mo
					, pfc_BDR.marca ma
					, pfc_BDR.centre ce
					, pfc_BDR.empresa em
					, pfc_BDR.poblacio p 
			WHERE m.idequip = e.idequip
					  AND e.dataBaixa is null
					  AND e.idcategoria = ca.idcategoria
					  AND e.idmodel = mo.idmodel
					  AND mo.idmarca = ma.idmarca
					  AND e.idcentre = ce.idcentre
					  AND m.idempresa = em.idempresa
					  AND em.idpoblacio = p.idpoblacio(+)
			ORDER BY em.nomempresa, m.datainici    
			;
	BEGIN          
	  FOR regMants IN cManteniments LOOP
	    PIPE ROW(regMants);
	  END LOOP;
  
      RETURN;
	END;
	
	

END;
/

COMMIT;

spool off
exit
