// Copyright  2003 Cognos Incorporated. All Rights Reserved.
// Cognos and the Cognos logo are trademarks of Cognos Incorporated.

/// htmltoc.js
///
/// Functions to help manage the table of contents.


// TODO -- change the style of the selected element.

function alert_obj(obj)
{
  var msg = typeof(obj) + "\n";
  msg = msg + obj + "\n";
  var name;
  for (name in obj) {
    msg = msg + name + ":" + ":" + obj[name] + "\n";
  }
  alert(msg);
}


/// Have we completed on initialization.
var loaded = false;

/// Map of hashes's to the link that defines them
var hash_to_link;
/// Map of indexes to hashes
var index_to_hash;
/// Map of hashes to indexes
var hash_to_index;

/// currently selected entry (ID)
var current_selection_id = null;



/// Find the report contents window.
function win_report_contents()
{
  if (window.parent) {
    if (window.parent.frames) {
	  return window.parent.frames["body"];
	}
  }
  return null;
}

/// Find the navigation buttons window.
function win_buttons()
{
  if (window.parent) {
    if (window.parent.frames) {
	  return window.parent.frames["navbuttons"];
	}
  }
  return null;
}
    


/// Return the link for an index.
function index_to_link(index)
{
  var hash;
  if (index < 0) {
    return null;
  }
  if (index >= index_to_hash.length) {
    return null;
  }
  hash = index_to_hash[index];
  if (hash == null) {
    return null;
  }
  var aLink = hash_to_link[hash];
  return aLink;
}




/// Move to the next entry.
function next()
{
  if (current_selection_id != null) {
    var index = hash_to_index[current_selection_id];
    index++;
    var aLink = index_to_link(index);
    if (aLink) {
      self.location = aLink.href;
      window.location.hash = "#" + aLink.id;
    }
  }
}
	  


/// Move to the previous entry.
function previous()
{
  if (current_selection_id != null) {
    var index = hash_to_index[current_selection_id];
    index--;
    var aLink = index_to_link(index);
    if (aLink) {
      self.location = aLink.href;
      window.location.hash = "#" + aLink.id;
    }
  }
}


/// Update the button status
function update_buttons()
{
  var buttons = win_buttons();
  if (buttons == null) {
    return;
  }

  if (current_selection_id == null) {
    buttons.enable_up_button(false);
    buttons.enable_down_button(false);
    buttons.enable_home_button(false);
    buttons.enable_end_button(false);
    return;
  }
  var index = hash_to_index[current_selection_id]
  if (index == null) {
    // something is wrong.
    buttons.enable_up_button(false);
    buttons.enable_down_button(false);
    buttons.enable_home_button(false);
    buttons.enable_end_button(false);
    return;
  }

  if (index <= 0) {
    buttons.enable_up_button(false);
    buttons.enable_home_button(false);
  } else {
    buttons.enable_up_button(true);
    buttons.enable_home_button(true);
  }
  var upper_bound = hash_to_index.length - 1;
  if ( index >= upper_bound) {
    buttons.enable_down_button(false);
    buttons.enable_end_button(false);
  } else {
    buttons.enable_down_button(true);
    buttons.enable_end_button(true);
  }
}

function hoverOver(obj)
{
	if (obj.id == current_selection_id)
	{
		obj.className="listItemActiveHover";
	}
	else
	{
		if (obj.className == "listItemVisited")
		{
			obj.className="listItemHoverVisited";
		}
		else
		{
			obj.className="listItemHover";
		}
	}

}

function hoverOut(obj)
{
	if (obj.id == current_selection_id)
	{
			obj.className="listItemActive";	
	}
	else
	{
		if (obj.className == "listItemHoverVisited")
		{
			obj.className="listItemVisited";
		}
		else
		{
			obj.className="listItem";
		}			
		
	}
}

	  
/// Remove the current highlight and set it to the new ID
/// if it is not null.
/// Return true if the id is valid.
function highlight(newId)
{
  var span; // an HTMLElement for the span encompassing a TOC entry.
  var style; // a Style object

  var result = true;

  if (newId != current_selection_id) {
    if (current_selection_id != null) {
	  if (document) {
	    if (document.getElementById) {
		  span = document.getElementById(current_selection_id);
		} else if (document.all) {
          span = document.all.item(current_selection_id);
		}
		if (span)
          if (span.style) {
 			span.className="listItemVisited";  
        }			
      }
    }

    current_selection_id = newId;

    var index = hash_to_index[newId];
    if (index == null) {
      result = false;
    }

    if (current_selection_id) {
	  if (document) {
	    var new_span = null;
	    if (document.getElementById) {
		  new_span = document.getElementById(current_selection_id);
		} else if (document.all) {
          new_span = document.all.item(current_selection_id);
		}
        if (new_span) {
          if (new_span.style) {
			new_span.className="listItemActive";  
          }			
		}
      }
    }
  }

  update_buttons();

  return result;
}


/// Change a window and its frames contents
/// file#hash -- new contents
function change_window_contents(win, file, hash)
{ 
  var nbFrames;
  var frameNb;
  var frame;
  var url = "";
  if (file) {
    url = file;
  }
  if (hash) {
    url = url + "#" + hash;
  }

  win.location = url;
    // The main window, and its sub-frames are reloaded.
  
  // On IE onload is not called when the same file is being loaded.
  // This recursive call sets the IFrames to the right location.
  if (hash) {
    nbFrames = win.frames.length;
    frameNb = 0;
    while (frameNb < nbFrames) {
      frame = win.frames[frameNb];   
      var path = frame.location.pathname;
      if (frame.location.search) {
        path = frame.location.search;
      }
      change_window_contents(frame, path, hash);
      frameNb++;
    }
  }
}


/// Change the viewed contents.
function change_contents(url)
{
  var contents = win_report_contents();
  if (! contents) {
    return;
  } 

  var url_parts = url.split('#');
  change_window_contents(contents, url_parts[0], url_parts[1]);  

  // On IE -- onLoad is not called when the same file is being loaded, so we
  // have to set the highlight ourselves.
  highlight(url_parts[1]);
}



/// Function called by the contents page to perform initialization.
function onLoad()
{
  var nbLinks = document.links.length;

  hash_to_link = new Array(nbLinks);
  hash_to_index = new Array(nbLinks);
  index_to_hash = new Array(nbLinks);

  var linkNb = 0;
  while (linkNb < nbLinks) {
    var link = document.links[linkNb];
    var hash = link.id
    hash_to_link[hash] = link;
    hash_to_index[hash] = linkNb;
    index_to_hash[linkNb] = hash;
    linkNb++;
  }

  loaded = true;

  var contents = win_report_contents();
  if (contents) {
    if (contents.loaded) {
      // The contents got loaded before we did.  Have the page
      // call us pack with the correct highlight.
      contents.update_navigation();
    }
  }
}
  
// Function called to home to the top of the TOC
function onHome()
{
  var aLink = index_to_link(0);
  if (aLink) {
      self.location = aLink.href;
      window.location.hash = "#" + aLink.id;
    }
}

// Function called to jump to the bottom of the TOC
function onEnd()
{
  var l = hash_to_index.length-1;
  var aLink = index_to_link(l);
  if (aLink) {
      self.location = aLink.href;
      window.location.hash = "#" + aLink.id;
    }
}